# Copyright (c) 2016 Juniper Networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.cli import format_columns
from osc_lib.cli.parseractions import KeyValueAction
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.osc import utils as nc_osc_utils
from neutronclient.osc.v2.networking_bgpvpn import constants

LOG = logging.getLogger(__name__)

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
    ('name', 'Name', column_util.LIST_BOTH),
    ('type', 'Type', column_util.LIST_BOTH),
    ('route_targets', 'Route Targets', column_util.LIST_LONG_ONLY),
    ('import_targets', 'Import Targets', column_util.LIST_LONG_ONLY),
    ('export_targets', 'Export Targets', column_util.LIST_LONG_ONLY),
    ('route_distinguishers', 'Route Distinguishers',
     column_util.LIST_LONG_ONLY),
    ('networks', 'Associated Networks', column_util.LIST_LONG_ONLY),
    ('routers', 'Associated Routers', column_util.LIST_LONG_ONLY),
    ('ports', 'Associated Ports', column_util.LIST_LONG_ONLY),
    ('vni', 'VNI', column_util.LIST_LONG_ONLY),
    ('local_pref', 'Local Pref', column_util.LIST_LONG_ONLY),
)
_formatters = {
    'route_targets': format_columns.ListColumn,
    'import_targets': format_columns.ListColumn,
    'export_targets': format_columns.ListColumn,
    'route_distinguishers': format_columns.ListColumn,
    'networks': format_columns.ListColumn,
    'routers': format_columns.ListColumn,
    'ports': format_columns.ListColumn,
}


def _get_common_parser(parser, update=None):
    """Adds to parser arguments common to create, set and unset commands.

    :params ArgumentParser parser: argparse object contains all command's
                                   arguments
    :params string update: Determines if it is a create command (value: None),
                           it is a set command (value: 'set') or if it is an
                           unset command (value: 'unset')
    """
    ADD_RT = _("Add Route Target to import/export list")
    REMOVE_RT = _("Remove Route Target from import/export list")
    ADD_IMPORT_RT = _("Add Route Target to import list")
    REMOVE_IMPORT_RT = _("Remove Route Target from import list")
    ADD_EXPORT_RT = _("Add Route Target to export list")
    REMOVE_EXPORT_RT = _("Remove Route Target from export list")
    ADD_RD = _("Add Route Distinguisher to the list of Route Distinguishers "
               "from which a Route Distinguishers will be picked from to "
               "advertise a VPN route")
    REMOVE_RD = _("Remove Route Distinguisher from the list of Route "
                  "Distinguishers from which a Route Distinguishers will be "
                  "picked from to advertise a VPN route")
    REPEAT_RT = _("repeat option for multiple Route Targets")
    REPEAT_RD = _("repeat option for multiple Route Distinguishers")

    def is_appended():
        return update is None or update == 'set'

    if update is None or update == 'set':
        parser.add_argument(
            '--name',
            metavar="<name>",
            help=_("Name of the BGP VPN"),
        )
    parser.add_argument(
        '--route-target',
        dest='route_targets',
        action='append',
        metavar="<route-target>",
        help="%s (%s)" % ((ADD_RT if is_appended() else REMOVE_RT), REPEAT_RT),
    )
    if update:
        parser.add_argument(
            '--no-route-target' if update == 'set' else '--all-route-target',
            dest='purge_route_target',
            action='store_true',
            help=_('Empty route target list'),
        )
    parser.add_argument(
        '--import-target',
        dest='import_targets',
        action='append',
        metavar="<import-target>",
        help="%s (%s)" % ((ADD_IMPORT_RT if is_appended() else
                           REMOVE_IMPORT_RT), REPEAT_RT),
    )
    if update:
        parser.add_argument(
            '--no-import-target' if update == 'set' else '--all-import-target',
            dest='purge_import_target',
            action='store_true',
            help=_('Empty import route target list'),
        )
    parser.add_argument(
        '--export-target',
        dest='export_targets',
        action='append',
        metavar="<export-target>",
        help="%s (%s)" % ((ADD_EXPORT_RT if is_appended() else
                           REMOVE_EXPORT_RT), REPEAT_RT),
    )
    if update:
        parser.add_argument(
            '--no-export-target' if update == 'set' else
            '--all-export-target',
            dest='purge_export_target',
            action='store_true',
            help=_('Empty export route target list'),
        )
    parser.add_argument(
        '--route-distinguisher',
        dest='route_distinguishers',
        action='append',
        metavar="<route-distinguisher>",
        help="%s (%s)" % ((ADD_RD if is_appended() else REMOVE_RD), REPEAT_RD),
    )
    if update:
        parser.add_argument(
            '--no-route-distinguisher' if update == 'set' else
            '--all-route-distinguisher',
            dest='purge_route_distinguisher',
            action='store_true',
            help=_('Empty route distinguisher list'),
        )
    parser.add_argument(
        '--vni', type=int,
        help=_('VXLAN Network Identifier to be used for this BGPVPN '
               'when a VXLAN encapsulation is used'))
    parser.add_argument(
        '--local-pref', type=int,
        dest='local_pref',
        help=_('Default BGP LOCAL_PREF to use in route advertisements'
               'towards this BGPVPN.'))


def _args2body(client_manager, id, action, args):

    if (not (args.purge_route_target and args.purge_import_target and
             args.purge_export_target and args.purge_route_distinguisher) and
            (args.route_targets or args.import_targets or
             args.export_targets or args.route_distinguishers)):
        bgpvpn = client_manager.neutronclient.show_bgpvpn(id)['bgpvpn']

    attrs = {}

    if 'name' in args and args.name is not None:
        attrs['name'] = str(args.name)

    if 'vni' in args and args.vni is not None:
        attrs['vni'] = args.vni

    if 'local_pref' in args and args.local_pref is not None:
        attrs['local_pref'] = args.local_pref

    if args.purge_route_target:
        attrs['route_targets'] = []
    elif args.route_targets:
        if action == 'set':
            attrs['route_targets'] = list(set(bgpvpn['route_targets']) |
                                          set(args.route_targets))
        elif action == 'unset':
            attrs['route_targets'] = list(set(bgpvpn['route_targets']) -
                                          set(args.route_targets))

    if args.purge_import_target:
        attrs['import_targets'] = []
    elif args.import_targets:
        if action == 'set':
            attrs['import_targets'] = list(set(bgpvpn['import_targets']) |
                                           set(args.import_targets))
        elif action == 'unset':
            attrs['import_targets'] = list(set(bgpvpn['import_targets']) -
                                           set(args.import_targets))

    if args.purge_export_target:
        attrs['export_targets'] = []
    elif args.export_targets:
        if action == 'set':
            attrs['export_targets'] = list(set(bgpvpn['export_targets']) |
                                           set(args.export_targets))
        elif action == 'unset':
            attrs['export_targets'] = list(set(bgpvpn['export_targets']) -
                                           set(args.export_targets))

    if args.purge_route_distinguisher:
        attrs['route_distinguishers'] = []
    elif args.route_distinguishers:
        if action == 'set':
            attrs['route_distinguishers'] = list(
                set(bgpvpn['route_distinguishers']) |
                set(args.route_distinguishers))
        elif action == 'unset':
            attrs['route_distinguishers'] = list(
                set(bgpvpn['route_distinguishers']) -
                set(args.route_distinguishers))

    return {constants.BGPVPN: attrs}


class CreateBgpvpn(command.ShowOne):
    _description = _("Create BGP VPN resource")

    def get_parser(self, prog_name):
        parser = super(CreateBgpvpn, self).get_parser(prog_name)
        nc_osc_utils.add_project_owner_option_to_parser(parser)
        _get_common_parser(parser)
        parser.add_argument(
            '--type',
            default='l3',
            choices=['l2', 'l3'],
            help=_("BGP VPN type selection between IP VPN (l3) and Ethernet "
                   "VPN (l2) (default: l3)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = {}
        if parsed_args.name is not None:
            attrs['name'] = str(parsed_args.name)
        if parsed_args.type is not None:
            attrs['type'] = parsed_args.type
        if parsed_args.route_targets is not None:
            attrs['route_targets'] = parsed_args.route_targets
        if parsed_args.import_targets is not None:
            attrs['import_targets'] = parsed_args.import_targets
        if parsed_args.export_targets is not None:
            attrs['export_targets'] = parsed_args.export_targets
        if parsed_args.route_distinguishers is not None:
            attrs['route_distinguishers'] = parsed_args.route_distinguishers
        if parsed_args.vni is not None:
            attrs['vni'] = parsed_args.vni
        if parsed_args.local_pref is not None:
            attrs['local_pref'] = parsed_args.local_pref
        if 'project' in parsed_args and parsed_args.project is not None:
            project_id = nc_osc_utils.find_project(
                self.app.client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            attrs['tenant_id'] = project_id
        body = {constants.BGPVPN: attrs}
        obj = client.create_bgpvpn(body)[constants.BGPVPN]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=_formatters)
        return display_columns, data


class SetBgpvpn(command.Command):
    _description = _("Set BGP VPN properties")

    def get_parser(self, prog_name):
        parser = super(SetBgpvpn, self).get_parser(prog_name)
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=_("BGP VPN to update (name or ID)"),
        )
        _get_common_parser(parser, update='set')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)['id']
        body = _args2body(self.app.client_manager, id, 'set', parsed_args)
        client.update_bgpvpn(id, body)


class UnsetBgpvpn(command.Command):
    _description = _("Unset BGP VPN properties")

    def get_parser(self, prog_name):
        parser = super(UnsetBgpvpn, self).get_parser(prog_name)
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=_("BGP VPN to update (name or ID)"),
        )
        _get_common_parser(parser, update='unset')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)['id']
        body = _args2body(self.app.client_manager, id, 'unset', parsed_args)
        client.update_bgpvpn(id, body)


class DeleteBgpvpn(command.Command):
    _description = _("Delete BGP VPN resource(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteBgpvpn, self).get_parser(prog_name)
        parser.add_argument(
            'bgpvpns',
            metavar="<bgpvpn>",
            nargs="+",
            help=_("BGP VPN(s) to delete (name or ID)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        fails = 0
        for id_or_name in parsed_args.bgpvpns:
            try:
                id = client.find_resource(constants.BGPVPN, id_or_name)['id']
                client.delete_bgpvpn(id)
                LOG.warning("BGP VPN %(id)s deleted", {'id': id})
            except Exception as e:
                fails += 1
                LOG.error("Failed to delete BGP VPN with name or ID "
                          "'%(id_or_name)s': %(e)s",
                          {'id_or_name': id_or_name, 'e': e})
        if fails > 0:
            msg = (_("Failed to delete %(fails)s of %(total)s BGP VPN.") %
                   {'fails': fails, 'total': len(parsed_args.bgpvpns)})
            raise exceptions.CommandError(msg)


class ListBgpvpn(command.Lister):
    _description = _("List BGP VPN resources")

    def get_parser(self, prog_name):
        parser = super(ListBgpvpn, self).get_parser(prog_name)
        nc_osc_utils.add_project_owner_option_to_parser(parser)
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output"),
        )
        parser.add_argument(
            '--property',
            metavar="<key=value>",
            default=dict(),
            help=_("Filter property to apply on returned BGP VPNs (repeat to "
                   "filter on multiple properties)"),
            action=KeyValueAction,
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        params = {}
        if parsed_args.project is not None:
            project_id = nc_osc_utils.find_project(
                self.app.client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            params['tenant_id'] = project_id
        if parsed_args.property:
            params.update(parsed_args.property)
        objs = client.list_bgpvpns(**params)[constants.BGPVPNS]
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers, (osc_utils.get_dict_properties(
            s, columns, formatters=_formatters) for s in objs))


class ShowBgpvpn(command.ShowOne):
    _description = _("Show information of a given BGP VPN")

    def get_parser(self, prog_name):
        parser = super(ShowBgpvpn, self).get_parser(prog_name)
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=_("BGP VPN to display (name or ID)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)['id']
        obj = client.show_bgpvpn(id)[constants.BGPVPN]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=_formatters)
        return display_columns, data



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _get_remote(rule):
    if rule['remote_ip_prefix']:
        remote = '%s (CIDR)' % rule['remote_ip_prefix']
    elif rule['remote_group_id']:
        remote = '%s (group)' % rule['remote_group_id']
    else:
        remote = None
    return remote


def _get_protocol_port(rule):
    proto = rule['protocol']
    port_min = rule['port_range_min']
    port_max = rule['port_range_max']
    if proto in ('tcp', 'udp'):
        if (port_min and port_min == port_max):
            protocol_port = '%s/%s' % (port_min, proto)
        elif port_min:
            protocol_port = '%s-%s/%s' % (port_min, port_max, proto)
        else:
            protocol_port = proto
    elif proto == 'icmp':
        icmp_opts = []
        if port_min is not None:
            icmp_opts.append('type:%s' % port_min)
        if port_max is not None:
            icmp_opts.append('code:%s' % port_max)

        if icmp_opts:
            protocol_port = 'icmp (%s)' % ', '.join(icmp_opts)
        else:
            protocol_port = 'icmp'
    elif proto is not None:
        # port_range_min/max are not recognized for protocol
        # other than TCP, UDP and ICMP.
        protocol_port = proto
    else:
        protocol_port = None

    return protocol_port


def _format_sg_rule(rule):
    formatted = []
    for field in ['direction',
                  'ethertype',
                  ('protocol_port', _get_protocol_port),
                  'remote_ip_prefix',
                  'remote_group_id']:
        if isinstance(field, tuple):
            field, get_method = field
            data = get_method(rule)
        else:
            data = rule[field]
        if not data:
            continue
        if field in ('remote_ip_prefix', 'remote_group_id'):
            data = '%s: %s' % (field, data)
        formatted.append(data)
    return ', '.join(formatted)


def _format_sg_rules(secgroup):
    try:
        return '\n'.join(sorted([_format_sg_rule(rule) for rule
                                 in secgroup['security_group_rules']]))
    except Exception:
        return ''


def generate_default_ethertype(protocol):
    if protocol == 'icmpv6':
        return 'IPv6'
    return 'IPv4'


class ListSecurityGroup(neutronV20.ListCommand):
    """List security groups that belong to a given tenant."""

    resource = 'security_group'
    list_columns = ['id', 'name', 'security_group_rules']
    _formatters = {'security_group_rules': _format_sg_rules}
    pagination_support = True
    sorting_support = True


class ShowSecurityGroup(neutronV20.ShowCommand):
    """Show information of a given security group."""

    resource = 'security_group'
    allow_names = True
    json_indent = 5


class CreateSecurityGroup(neutronV20.CreateCommand):
    """Create a security group."""

    resource = 'security_group'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the security group to be created.'))
        parser.add_argument(
            '--description',
            help=_('Description of the security group to be created.'))

    def args2body(self, parsed_args):
        body = {'name': parsed_args.name}
        neutronV20.update_dict(parsed_args, body,
                               ['description', 'tenant_id'])
        return {'security_group': body}


class DeleteSecurityGroup(neutronV20.DeleteCommand):
    """Delete a given security group."""

    resource = 'security_group'
    allow_names = True


class UpdateSecurityGroup(neutronV20.UpdateCommand):
    """Update a given security group."""

    resource = 'security_group'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Updated name of the security group.'))
        parser.add_argument(
            '--description',
            help=_('Updated description of the security group.'))

    def args2body(self, parsed_args):
        body = {}
        neutronV20.update_dict(parsed_args, body,
                               ['name', 'description'])
        return {'security_group': body}


class ListSecurityGroupRule(neutronV20.ListCommand):
    """List security group rules that belong to a given tenant."""

    resource = 'security_group_rule'
    list_columns = ['id', 'security_group_id', 'direction',
                    'ethertype', 'port/protocol', 'remote']
    # replace_rules: key is an attribute name in Neutron API and
    # corresponding value is a display name shown by CLI.
    replace_rules = {'security_group_id': 'security_group',
                     'remote_group_id': 'remote_group'}
    digest_fields = {
        # The entry 'protocol/port' is left deliberately for backwards
        # compatibility.
        'remote': {
            'method': _get_remote,
            'depends_on': ['remote_ip_prefix', 'remote_group_id']},
        'port/protocol': {
            'method': _get_protocol_port,
            'depends_on': ['protocol', 'port_range_min', 'port_range_max']},
        'protocol/port': {
            'method': _get_protocol_port,
            'depends_on': ['protocol', 'port_range_min', 'port_range_max']}}
    pagination_support = True
    sorting_support = True

    def get_parser(self, prog_name):
        parser = super(ListSecurityGroupRule, self).get_parser(prog_name)
        parser.add_argument(
            '--no-nameconv', action='store_true',
            help=_('Do not convert security group ID to its name.'))
        return parser

    @staticmethod
    def replace_columns(cols, rules, reverse=False):
        if reverse:
            rules = dict((rules[k], k) for k in rules.keys())
        return [rules.get(col, col) for col in cols]

    def get_required_fields(self, fields):
        fields = self.replace_columns(fields, self.replace_rules, reverse=True)
        for field, digest_fields in self.digest_fields.items():
            if field in fields:
                fields += digest_fields['depends_on']
                fields.remove(field)
        return fields

    def retrieve_list(self, parsed_args):
        parsed_args.fields = self.get_required_fields(parsed_args.fields)
        return super(ListSecurityGroupRule, self).retrieve_list(parsed_args)

    def _get_sg_name_dict(self, data, page_size, no_nameconv):
        """Get names of security groups referred in the retrieved rules.

        :return: a dict from secgroup ID to secgroup name
        """
        if no_nameconv:
            return {}
        neutron_client = self.get_client()
        search_opts = {'fields': ['id', 'name']}
        if self.pagination_support:
            if page_size:
                search_opts.update({'limit': page_size})
        sec_group_ids = set()
        for rule in data:
            for key in self.replace_rules:
                if rule.get(key):
                    sec_group_ids.add(rule[key])
        sec_group_ids = list(sec_group_ids)

        def _get_sec_group_list(sec_group_ids):
            search_opts['id'] = sec_group_ids
            return neutron_client.list_security_groups(
                **search_opts).get('security_groups', [])

        try:
            secgroups = _get_sec_group_list(sec_group_ids)
        except exceptions.RequestURITooLong as uri_len_exc:
            # Length of a query filter on security group rule id
            # id=<uuid>& (with len(uuid)=36)
            sec_group_id_filter_len = 40
            # The URI is too long because of too many sec_group_id filters
            # Use the excess attribute of the exception to know how many
            # sec_group_id filters can be inserted into a single request
            sec_group_count = len(sec_group_ids)
            max_size = ((sec_group_id_filter_len * sec_group_count) -
                        uri_len_exc.excess)
            chunk_size = max_size // sec_group_id_filter_len
            secgroups = []
            for i in range(0, sec_group_count, chunk_size):
                secgroups.extend(
                    _get_sec_group_list(sec_group_ids[i: i + chunk_size]))

        return dict([(sg['id'], sg['name'])
                     for sg in secgroups if sg['name']])

    @staticmethod
    def _has_fields(rule, required_fields):
        return all([key in rule for key in required_fields])

    def extend_list(self, data, parsed_args):
        sg_dict = self._get_sg_name_dict(data, parsed_args.page_size,
                                         parsed_args.no_nameconv)
        for rule in data:
            # Replace security group UUID with its name.
            for key in self.replace_rules:
                if key in rule:
                    rule[key] = sg_dict.get(rule[key], rule[key])
            for field, digest_rule in self.digest_fields.items():
                if self._has_fields(rule, digest_rule['depends_on']):
                    rule[field] = digest_rule['method'](rule) or 'any'

    def setup_columns(self, info, parsed_args):
        # Translate the specified columns from the command line
        # into field names used in "info".
        parsed_args.columns = self.replace_columns(parsed_args.columns,
                                                   self.replace_rules,
                                                   reverse=True)
        # NOTE(amotoki): 2nd element of the tuple returned by setup_columns()
        # is a generator, so if you need to create a look using the generator
        # object, you need to recreate a generator to show a list expectedly.
        info = super(ListSecurityGroupRule, self).setup_columns(info,
                                                                parsed_args)
        cols = info[0]
        if not parsed_args.no_nameconv:
            # Replace column names in the header line (in info[0])
            cols = self.replace_columns(info[0], self.replace_rules)
            parsed_args.columns = cols
        return (cols, info[1])


class ShowSecurityGroupRule(neutronV20.ShowCommand):
    """Show information of a given security group rule."""

    resource = 'security_group_rule'
    allow_names = False


class CreateSecurityGroupRule(neutronV20.CreateCommand):
    """Create a security group rule."""

    resource = 'security_group_rule'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description of security group rule.'))
        parser.add_argument(
            'security_group_id', metavar='SECURITY_GROUP',
            help=_('ID or name of the security group to '
                   'which the rule is added.'))
        parser.add_argument(
            '--direction',
            type=utils.convert_to_lowercase,
            default='ingress', choices=['ingress', 'egress'],
            help=_('Direction of traffic: ingress/egress.'))
        parser.add_argument(
            '--ethertype',
            help=_('IPv4/IPv6'))
        parser.add_argument(
            '--protocol',
            type=utils.convert_to_lowercase,
            help=_('Protocol of packet. Allowed values are '
                   '[icmp, icmpv6, tcp, udp] and '
                   'integer representations [0-255].'))
        parser.add_argument(
            '--port-range-min',
            help=_('Starting port range. For ICMP it is type.'))
        parser.add_argument(
            '--port_range_min',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--port-range-max',
            help=_('Ending port range. For ICMP it is code.'))
        parser.add_argument(
            '--port_range_max',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--remote-ip-prefix',
            help=_('CIDR to match on.'))
        parser.add_argument(
            '--remote_ip_prefix',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--remote-group-id', metavar='REMOTE_GROUP',
            help=_('ID or name of the remote security group '
                   'to which the rule is applied.'))
        parser.add_argument(
            '--remote_group_id',
            help=argparse.SUPPRESS)

    def args2body(self, parsed_args):
        _security_group_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'security_group', parsed_args.security_group_id)
        body = {'security_group_id': _security_group_id,
                'direction': parsed_args.direction,
                'ethertype': parsed_args.ethertype or
                generate_default_ethertype(parsed_args.protocol)}
        neutronV20.update_dict(parsed_args, body,
                               ['protocol', 'port_range_min', 'port_range_max',
                                'remote_ip_prefix', 'tenant_id',
                                'description'])
        if parsed_args.remote_group_id:
            _remote_group_id = neutronV20.find_resourceid_by_name_or_id(
                self.get_client(), 'security_group',
                parsed_args.remote_group_id)
            body['remote_group_id'] = _remote_group_id
        return {'security_group_rule': body}


class DeleteSecurityGroupRule(neutronV20.DeleteCommand):
    """Delete a given security group rule."""

    resource = 'security_group_rule'
    allow_names = False



# Copyright 2016 Radware LTD.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _get_policy_id(client, policy_id_or_name):
    return neutronV20.find_resourceid_by_name_or_id(
        client, 'l7policy', policy_id_or_name,
        cmd_resource='lbaas_l7policy')


class LbaasL7RuleMixin(object):

    def set_extra_attrs(self, parsed_args):
        self.parent_id = _get_policy_id(self.get_client(),
                                        parsed_args.l7policy)

    def add_known_arguments(self, parser):
        parser.add_argument(
            'l7policy', metavar='L7POLICY',
            help=_('ID or name of L7 policy this rule belongs to.'))


def _add_common_args(parser, is_create=True):
    parser.add_argument(
        '--type',
        required=is_create,
        type=utils.convert_to_uppercase,
        choices=['HOST_NAME', 'PATH', 'FILE_TYPE', 'HEADER', 'COOKIE'],
        help=_('Rule type.'))
    parser.add_argument(
        '--compare-type',
        required=is_create,
        type=utils.convert_to_uppercase,
        choices=['REGEX', 'STARTS_WITH', 'ENDS_WITH',
                 'CONTAINS', 'EQUAL_TO'],
        help=_('Rule compare type.'))
    parser.add_argument(
        '--invert-compare',
        dest='invert',
        action='store_true',
        help=_('Invert the compare type.'))
    parser.add_argument(
        '--key',
        help=_('Key to compare.'
               ' Relevant for HEADER and COOKIE types only.'))
    parser.add_argument(
        '--value',
        required=is_create,
        help=_('Value to compare.'))


def _common_args2body(client, parsed_args, is_create=True):
    attributes = ['type', 'compare_type',
                  'invert', 'key', 'value', 'admin_state_up']
    if is_create:
        attributes.append('tenant_id')
    body = {}
    neutronV20.update_dict(parsed_args, body, attributes)
    return {'rule': body}


class ListL7Rule(LbaasL7RuleMixin, neutronV20.ListCommand):
    """LBaaS v2 List L7 rules that belong to a given L7 policy."""

    resource = 'rule'
    shadow_resource = 'lbaas_l7rule'
    pagination_support = True
    sorting_support = True

    list_columns = [
        'id', 'type', 'compare_type', 'invert', 'key', 'value',
        'admin_state_up', 'status'
    ]

    def take_action(self, parsed_args):
        self.parent_id = _get_policy_id(self.get_client(),
                                        parsed_args.l7policy)
        self.values_specs.append('--l7policy_id=%s' % self.parent_id)
        return super(ListL7Rule, self).take_action(parsed_args)


class ShowL7Rule(LbaasL7RuleMixin, neutronV20.ShowCommand):
    """LBaaS v2 Show information of a given rule."""

    resource = 'rule'
    shadow_resource = 'lbaas_l7rule'


class CreateL7Rule(LbaasL7RuleMixin, neutronV20.CreateCommand):
    """LBaaS v2 Create L7 rule."""

    resource = 'rule'
    shadow_resource = 'lbaas_l7rule'

    def add_known_arguments(self, parser):
        super(CreateL7Rule, self).add_known_arguments(parser)
        _add_common_args(parser)
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state_up',
            action='store_false',
            help=_('Set admin state up to false'))

    def args2body(self, parsed_args):
        return _common_args2body(self.get_client(), parsed_args)


class UpdateL7Rule(LbaasL7RuleMixin, neutronV20.UpdateCommand):
    """LBaaS v2 Update a given L7 rule."""

    resource = 'rule'
    shadow_resource = 'lbaas_l7rule'

    def add_known_arguments(self, parser):
        super(UpdateL7Rule, self).add_known_arguments(parser)
        _add_common_args(parser, False)
        utils.add_boolean_argument(
            parser, '--admin-state-up',
            help=_('Specify the administrative state of the rule'
                   ' (True meaning "Up").'))

    def args2body(self, parsed_args):
        return _common_args2body(self.get_client(), parsed_args, False)


class DeleteL7Rule(LbaasL7RuleMixin, neutronV20.DeleteCommand):
    """LBaaS v2 Delete a given L7 rule."""

    resource = 'rule'
    shadow_resource = 'lbaas_l7rule'



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20


def add_common_args(parser):
    parser.add_argument(
        '--name',
        help=_('Name for the VPN service.'))
    parser.add_argument(
        '--description',
        help=_('Description for the VPN service.'))


def common_args2body(parsed_args, body):
    neutronv20.update_dict(parsed_args, body,
                           ['name', 'description'])


class ListVPNService(neutronv20.ListCommand):
    """List VPN service configurations that belong to a given tenant."""

    resource = 'vpnservice'
    list_columns = [
        'id', 'name', 'router_id', 'status'
    ]
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowVPNService(neutronv20.ShowCommand):
    """Show information of a given VPN service."""

    resource = 'vpnservice'
    help_resource = 'VPN service'


class CreateVPNService(neutronv20.CreateCommand):
    """Create a VPN service."""
    resource = 'vpnservice'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            'router', metavar='ROUTER',
            help=_('Router unique identifier for the VPN service.'))
        parser.add_argument(
            'subnet', nargs='?', metavar='SUBNET',
            help=_('[DEPRECATED in Mitaka] Unique identifier for the local '
                   'private subnet.'))
        add_common_args(parser)

    def args2body(self, parsed_args):
        if parsed_args.subnet:
            _subnet_id = neutronv20.find_resourceid_by_name_or_id(
                self.get_client(), 'subnet', parsed_args.subnet)
        else:
            _subnet_id = None
        _router_id = neutronv20.find_resourceid_by_name_or_id(
            self.get_client(), 'router',
            parsed_args.router)

        body = {'subnet_id': _subnet_id,
                'router_id': _router_id,
                'admin_state_up': parsed_args.admin_state}
        neutronv20.update_dict(parsed_args, body,
                               ['tenant_id'])
        common_args2body(parsed_args, body)
        return {self.resource: body}


class UpdateVPNService(neutronv20.UpdateCommand):
    """Update a given VPN service."""

    resource = 'vpnservice'
    help_resource = 'VPN service'

    def add_known_arguments(self, parser):
        add_common_args(parser)
        utils.add_boolean_argument(
            parser, '--admin-state-up',
            help=_('Update the admin state for the VPN Service.'
                   '(True means UP)'))

    def args2body(self, parsed_args):
        body = {}
        common_args2body(parsed_args, body)
        neutronv20.update_dict(parsed_args, body,
                               ['admin_state_up'])
        return {self.resource: body}


class DeleteVPNService(neutronv20.DeleteCommand):
    """Delete a given VPN service."""

    resource = 'vpnservice'
    help_resource = 'VPN service'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient.tests.functional import base


class SimpleReadOnlyNeutronVpnClientTest(base.ClientTestBase):

    """Tests for vpn based client commands that are read only

    This is a first pass at a simple read only python-neutronclient test.
    This only exercises vpn based client commands that are read only.
    This should test commands:
    * as a regular user
    * as a admin user
    * with and without optional parameters
    * initially just check return codes, and later test command outputs
    """
    def setUp(self):
        super(SimpleReadOnlyNeutronVpnClientTest, self).setUp()
        if not self.is_extension_enabled('vpnaas'):
            self.skipTest('VPNaaS is not enabled')

    def test_neutron_vpn_ikepolicy_list(self):
        ikepolicy = self.parser.listing(self.neutron('vpn-ikepolicy-list'))
        self.assertTableStruct(ikepolicy, ['id', 'name',
                                           'auth_algorithm',
                                           'encryption_algorithm',
                                           'ike_version', 'pfs'])

    def test_neutron_vpn_ipsecpolicy_list(self):
        ipsecpolicy = self.parser.listing(self.neutron('vpn-ipsecpolicy-list'))
        self.assertTableStruct(ipsecpolicy, ['id', 'name',
                                             'auth_algorithm',
                                             'encryption_algorithm',
                                             'pfs'])

    def test_neutron_vpn_service_list(self):
        vpn_list = self.parser.listing(self.neutron('vpn-service-list'))
        self.assertTableStruct(vpn_list, ['id', 'name',
                                          'router_id', 'status'])

    def test_neutron_ipsec_site_connection_list(self):
        ipsec_site = self.parser.listing(self.neutron
                                         ('ipsec-site-connection-list'))
        self.assertTableStruct(ipsec_site, ['id', 'name',
                                            'peer_address',
                                            'auth_mode', 'status'])



# Copyright 2014 Blue Box Group, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.lb.v2 import healthmonitor
from neutronclient.tests.unit import test_cli20


class CLITestV20LbHealthMonitorJSON(test_cli20.CLITestV20Base):

    def test_create_healthmonitor_with_mandatory_params(self):
        # lbaas-healthmonitor-create with mandatory params only.
        resource = 'healthmonitor'
        cmd_resource = 'lbaas_healthmonitor'
        cmd = healthmonitor.CreateHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        my_id = 'my-id'
        type = 'PING'
        max_retries = '3'
        delay = '10'
        timeout = '60'
        pool = 'pool1'
        args = ['--type', type, '--max-retries', max_retries,
                '--delay', delay, '--timeout', timeout, '--pool', pool]
        position_names = ['type', 'max_retries', 'delay', 'timeout', 'pool_id']
        position_values = [type, max_retries, delay, timeout, pool]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_create_healthmonitor_with_all_params(self):
        # lbaas-healthmonitor-create with all params set.
        resource = 'healthmonitor'
        cmd_resource = 'lbaas_healthmonitor'
        cmd = healthmonitor.CreateHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        my_id = 'my-id'
        type = 'PING'
        max_retries = '3'
        delay = '10'
        timeout = '60'
        http_method = 'GET'
        expected_codes = '201'
        url_path = '/somepath'
        pool = 'pool1'
        name = 'healthmonitor1'
        args = ['--admin-state-down', '--http-method', http_method,
                '--expected-codes', expected_codes, '--url-path', url_path,
                '--type', type, '--max-retries', max_retries,
                '--delay', delay, '--timeout', timeout, '--pool', pool,
                '--name', name]
        position_names = ['admin_state_up', 'http_method', 'expected_codes',
                          'url_path', 'type', 'max_retries', 'delay',
                          'timeout', 'pool_id', 'name']
        position_values = [False, http_method, expected_codes, url_path,
                           type, max_retries, delay, timeout, pool, name]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_list_healthmonitors(self):
        # lbaas-healthmonitor-list.
        resources = 'healthmonitors'
        cmd_resources = 'lbaas_healthmonitors'
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_healthmonitors_pagination(self):
        # lbaas-healthmonitor-list with pagination.
        resources = 'healthmonitors'
        cmd_resources = 'lbaas_healthmonitors'
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources_with_pagination(resources, cmd,
                                                  cmd_resources=cmd_resources)

    def test_list_healthmonitors_sort(self):
        # lbaas-healthmonitor-list --sort-key id --sort-key asc.
        resources = 'healthmonitors'
        cmd_resources = 'lbaas_healthmonitors'
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_healthmonitors_limit(self):
        # lbaas-healthmonitor-list -P.
        resources = 'healthmonitors'
        cmd_resources = 'lbaas_healthmonitors'
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, page_size=1000,
                                  cmd_resources=cmd_resources)

    def test_show_healthmonitor_id(self):
        # lbaas-healthmonitor-show test_id.
        resource = 'healthmonitor'
        cmd_resource = 'lbaas_healthmonitor'
        cmd = healthmonitor.ShowHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'],
                                 cmd_resource=cmd_resource)

    def test_show_healthmonitor_id_name(self):
        # lbaas-healthmonitor-show.
        resource = 'healthmonitor'
        cmd_resource = 'lbaas_healthmonitor'
        cmd = healthmonitor.ShowHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'],
                                 cmd_resource=cmd_resource)

    def _test_update_hm(self, args, expected_values):
        resource = 'healthmonitor'
        cmd_resource = 'lbaas_healthmonitor'
        my_id = 'myid'
        cmd = healthmonitor.UpdateHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        args.insert(0, my_id)
        self._test_update_resource(resource, cmd, my_id,
                                   args,
                                   expected_values,
                                   cmd_resource=cmd_resource)

    def test_update_healthmonitor(self):
        # lbaas-healthmonitor-update myid --name newname.
        self._test_update_hm(['--name', 'newname'], {'name': 'newname', })
        # lbaas-healthmonitor-update myid --delay 10.
        self._test_update_hm(['--delay', '10'], {'delay': '10'})
        # lbaas-healthmonitor-update myid --timeout 5.
        self._test_update_hm(['--timeout', '5'], {'timeout': '5', })
        # lbaas-healthmonitor-update myid --delay 10.
        self._test_update_hm(['--http-method', 'OPTIONS'],
                             {'http_method': 'OPTIONS'})
        # lbaas-healthmonitor-update myid --url-path /test/string .
        self._test_update_hm(['--url-path', '/test/string'],
                             {'url_path': '/test/string', })
        # lbaas-healthmonitor-update myid --max-retries 5
        self._test_update_hm(['--max-retries', '5'], {'max_retries': '5'})
        # lbaas-healthmonitor-update myid --expected-codes 201
        self._test_update_hm(['--expected-codes', '201'],
                             {'expected_codes': '201'})
        # lbaas-healthmonitor-update myid --admin-state-up False
        self._test_update_hm(['--admin-state-up', 'False'],
                             {'admin_state_up': 'False'})

    def test_delete_healthmonitor(self):
        # lbaas-healthmonitor-delete my-id.
        resource = 'healthmonitor'
        cmd_resource = 'lbaas_healthmonitor'
        cmd = healthmonitor.DeleteHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args,
                                   cmd_resource=cmd_resource)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from unittest import mock

from neutronclient.osc.v2.dynamic_routing import bgp_dragent
from neutronclient.tests.unit.osc.v2.dynamic_routing import fakes


class TestAddBgpSpeakerToDRAgent(fakes.TestNeutronDynamicRoutingOSCV2):
    _bgp_speaker = fakes.FakeBgpSpeaker.create_one_bgp_speaker()
    _bgp_dragent = fakes.FakeDRAgent.create_one_dragent()
    _bgp_speaker_id = _bgp_speaker['id']
    _bgp_dragent_id = _bgp_dragent['id']

    def setUp(self):
        super(TestAddBgpSpeakerToDRAgent, self).setUp()

        # Get the command object to test
        self.cmd = bgp_dragent.AddBgpSpeakerToDRAgent(self.app, self.namespace)

    def test_add_bgp_speaker_to_dragent(self):
        arglist = [
            self._bgp_dragent_id,
            self._bgp_speaker_id,
        ]
        verifylist = [
            ('dragent_id', self._bgp_dragent_id),
            ('bgp_speaker', self._bgp_speaker_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch.object(self.neutronclient,
                               "add_bgp_speaker_to_dragent",
                               return_value=None):

            result = self.cmd.take_action(parsed_args)
            self.neutronclient.add_bgp_speaker_to_dragent.\
                assert_called_once_with(
                    self._bgp_dragent_id,
                    {'bgp_speaker_id': self._bgp_speaker_id})
            self.assertIsNone(result)


class TestRemoveBgpSpeakerFromDRAgent(fakes.TestNeutronDynamicRoutingOSCV2):
    _bgp_speaker = fakes.FakeBgpSpeaker.create_one_bgp_speaker()
    _bgp_dragent = fakes.FakeDRAgent.create_one_dragent()
    _bgp_speaker_id = _bgp_speaker['id']
    _bgp_dragent_id = _bgp_dragent['id']

    def setUp(self):
        super(TestRemoveBgpSpeakerFromDRAgent, self).setUp()

        # Get the command object to test
        self.cmd = bgp_dragent.RemoveBgpSpeakerFromDRAgent(
            self.app, self.namespace)

    def test_remove_bgp_speaker_from_dragent(self):
        arglist = [
            self._bgp_dragent_id,
            self._bgp_speaker_id,
        ]
        verifylist = [
            ('dragent_id', self._bgp_dragent_id),
            ('bgp_speaker', self._bgp_speaker_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch.object(self.neutronclient,
                               "remove_bgp_speaker_from_dragent",
                               return_value=None):
            result = self.cmd.take_action(parsed_args)
            self.neutronclient.remove_bgp_speaker_from_dragent.\
                assert_called_once_with(self._bgp_dragent_id,
                                        self._bgp_speaker_id)
            self.assertIsNone(result)


class TestListDRAgentsHostingBgpSpeaker(fakes.TestNeutronDynamicRoutingOSCV2):
    _bgp_speaker = fakes.FakeBgpSpeaker.create_one_bgp_speaker()
    _bgp_speaker_id = _bgp_speaker['id']
    attrs = {'bgp_speaker_id': _bgp_speaker_id}
    _bgp_dragents = fakes.FakeDRAgent.create_dragents(attrs)
    columns = ('ID', 'Host', 'State', 'Alive')
    data = [(_bgp_dragent['id'],
             _bgp_dragent['host'],
             _bgp_dragent['admin_state_up'],
             ':-)' if _bgp_dragent['alive'] else 'XXX')
            for _bgp_dragent in _bgp_dragents['agents']]

    def setUp(self):
        super(TestListDRAgentsHostingBgpSpeaker, self).setUp()

        # Get the command object to test
        self.cmd = bgp_dragent.ListDRAgent(self.app, self.namespace)

    def test_list_dragents_hosting_bgp_speaker(self):
        arglist = [
            '--bgp-speaker', self._bgp_speaker_id,
        ]
        verifylist = [
            ('bgp_speaker', self._bgp_speaker_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch.object(self.neutronclient,
                               "list_dragents_hosting_bgp_speaker",
                               return_value=self._bgp_dragents):
            columns, data = self.cmd.take_action(parsed_args)
            attrs = {'bgp_speaker': self._bgp_speaker_id}
            self.neutronclient.list_dragents_hosting_bgp_speaker.\
                assert_called_once_with(**attrs)
            self.assertEqual(self.columns, columns)
            self.assertListEqual(self.data, list(data))



# Copyright 2015 Huawei Technologies India Pvt Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.qos import rule as qos_rule
from neutronclient.tests.unit import test_cli20


class CLITestV20QoSRuleJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['bandwidth_limit_rule',
                                  'dscp_marking_rule',
                                  'minimum_bandwidth_rule']

    def setUp(self):
        super(CLITestV20QoSRuleJSON, self).setUp()

    def test_list_qos_rule_types(self):
        # qos_rule_types.
        resources = 'rule_types'
        cmd_resources = 'qos_rule_types'
        response_contents = [{'type': 'bandwidth_limit'},
                             {'type': 'dscp_marking'},
                             {'type': 'minimum_bandwidth'}]

        cmd = qos_rule.ListQoSRuleTypes(test_cli20.MyApp(sys.stdout),
                                        None)
        self._test_list_resources(resources, cmd, True,
                                  response_contents=response_contents,
                                  cmd_resources=cmd_resources)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutronclient.neutron.v2_0 import availability_zone as az
from neutronclient.tests.unit import test_cli20


class CLITestV20Agent(test_cli20.CLITestV20Base):
    def test_list_agents(self):
        contents = {'availability_zones': [{'name': 'zone1',
                                            'resource': 'network',
                                            'state': 'available'},
                                           {'name': 'zone2',
                                            'resource': 'router',
                                            'state': 'unavailable'}]}
        args = ['-f', 'json']
        resources = "availability_zones"

        cmd = az.ListAvailabilityZone(test_cli20.MyApp(sys.stdout), None)
        self._test_list_columns(cmd, resources, contents, args)



# Copyright 2016 Huawei Technologies India Pvt. Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.bgp import dragentscheduler as bgp_drsched
from neutronclient.tests.unit import test_cli20
from neutronclient.tests.unit import test_cli20_agentschedulers as test_as


BGP_DRAGENT_ID = 'bgp_dragent_id1'
BGP_SPEAKER = 'bgp_speaker_id1'


class CLITestV20DRAgentScheduler(test_as.CLITestV20AgentScheduler):

    def test_add_bgp_speaker_to_dragent(self):
        resource = 'agent'
        cmd = bgp_drsched.AddBGPSpeakerToDRAgent(
            test_cli20.MyApp(sys.stdout), None)
        args = (BGP_DRAGENT_ID, BGP_SPEAKER)
        body = {'bgp_speaker_id': BGP_SPEAKER}
        result = {'bgp_speaker_id': 'bgp_speaker_id', }
        self._test_add_to_agent(resource, cmd, args,
                                self.client.BGP_DRINSTANCES,
                                body, result)

    def test_remove_bgp_speaker_from_dragent(self):
        resource = 'agent'
        cmd = bgp_drsched.RemoveBGPSpeakerFromDRAgent(
            test_cli20.MyApp(sys.stdout), None)
        args = (BGP_DRAGENT_ID, BGP_SPEAKER)
        self._test_remove_from_agent(resource, cmd, args,
                                     self.client.BGP_DRINSTANCES)

    def test_list_bgp_speakers_on_dragent(self):
        resources = 'bgp_speakers'
        cmd = bgp_drsched.ListBGPSpeakersOnDRAgent(
            test_cli20.MyApp(sys.stdout), None)
        path = ((self.client.agent_path + self.client.BGP_DRINSTANCES) %
                BGP_DRAGENT_ID)
        self._test_list_resources(resources, cmd, base_args=[BGP_DRAGENT_ID],
                                  path=path)

    def test_list_dragents_hosting_bgp_speaker(self):
        resources = 'agent'
        cmd = bgp_drsched.ListDRAgentsHostingBGPSpeaker(
            test_cli20.MyApp(sys.stdout), None)
        path = ((self.client.bgp_speaker_path + self.client.BGP_DRAGENTS) %
                BGP_DRAGENT_ID)
        contents = {self.id_field: 'myid1', 'alive': True}
        self._test_list_resources(resources, cmd, base_args=[BGP_DRAGENT_ID],
                                  path=path, response_contents=contents)



# Copyright 2016 Huawei Technologies India Pvt. Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.bgp import speaker as bgp_speaker
from neutronclient.tests.unit import test_cli20


class CLITestV20BGPSpeakerJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['bgp_speaker']

    def test_create_bgp_speaker_with_minimal_options(self):
        # Create BGP Speaker with mandatory params.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.CreateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        name = 'my-name'
        my_id = 'my-id'
        local_asnum = '1'
        args = [name, '--local-as', local_asnum, ]
        position_names = ['name', 'local_as', 'ip_version']
        position_values = [name, local_asnum, 4]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_ipv4_bgp_speaker_with_all_params(self):
        # Create BGP Speaker with all params.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.CreateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        name = 'my-name'
        my_id = 'my-id'
        local_asnum = '1'
        args = [name,
                '--local-as', local_asnum,
                '--ip-version', '4',
                '--advertise-floating-ip-host-routes', 'True',
                '--advertise-tenant-networks', 'True']
        position_names = ['name', 'local_as', 'ip_version',
                          'advertise_floating_ip_host_routes',
                          'advertise_tenant_networks']
        position_values = [name, local_asnum, 4, 'True', 'True']
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_ipv6_bgp_speaker_with_all_params(self):
        # Create BGP Speaker with all params.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.CreateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        name = 'my-name'
        my_id = 'my-id'
        local_asnum = '65535'
        args = [name,
                '--local-as', local_asnum,
                '--ip-version', '6',
                '--advertise-floating-ip-host-routes', 'True',
                '--advertise-tenant-networks', 'True']
        position_names = ['name', 'local_as', 'ip_version',
                          'advertise_floating_ip_host_routes',
                          'advertise_tenant_networks']
        position_values = [name, local_asnum, 6, 'True', 'True']
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_bgp_speaker_with_invalid_min_local_asnum(self):
        # Create BGP Speaker with invalid minimum local-asnum.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.CreateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        name = 'my-name'
        my_id = 'my-id'
        local_asnum = '0'
        args = [name,
                '--local-as', local_asnum]
        position_names = ['name', 'local_as']
        position_values = [name, local_asnum]
        exc = self.assertRaises(exceptions.CommandError,
                                self._test_create_resource,
                                resource, cmd, name, my_id, args,
                                position_names, position_values)
        self.assertEqual('local-as "0" should be an integer [%s:%s].' %
                         (bgp_speaker.MIN_AS_NUM, bgp_speaker.MAX_AS_NUM),
                         str(exc))

    def test_create_bgp_speaker_with_invalid_max_local_asnum(self):
        # Create BGP Speaker with invalid maximum local-asnum.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.CreateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        name = 'my-name'
        my_id = 'my-id'
        local_asnum = '4294967296'
        args = [name,
                '--local-as', local_asnum]
        position_names = ['name', 'local_as', ]
        position_values = [name, local_asnum, ]
        exc = self.assertRaises(exceptions.CommandError,
                                self._test_create_resource,
                                resource, cmd, name, my_id, args,
                                position_names, position_values)
        self.assertEqual('local-as "4294967296" should be an '
                         'integer [%s:%s].' %
                         (bgp_speaker.MIN_AS_NUM, bgp_speaker.MAX_AS_NUM),
                         str(exc))

    def test_update_bgp_speaker(self):
        # Update BGP Speaker:
        # myid --advertise-tenant-networks True
        #      --advertise-floating-ip-host-routes False
        resource = 'bgp_speaker'
        cmd = bgp_speaker.UpdateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid',
                                    '--name', 'new-name',
                                    '--advertise-tenant-networks', 'True',
                                    '--advertise-floating-ip-host-routes',
                                    'False'],
                                   {'name': 'new-name',
                                    'advertise_tenant_networks': 'True',
                                    'advertise_floating_ip_host_routes':
                                    'False'})

    def test_update_bgp_speaker_exception(self):
        # Update BGP Speaker: myid.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.UpdateSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        self.assertRaises(exceptions.CommandError,
                          self._test_update_resource,
                          resource, cmd, 'myid', ['myid'], {})

    def test_list_bgp_speaker(self):
        # List all BGP Speakers.
        resources = "bgp_speakers"
        cmd = bgp_speaker.ListSpeakers(test_cli20.MyApp(sys.stdout),
                                       None)
        self._test_list_resources(resources, cmd, True)

    @mock.patch.object(bgp_speaker.ListSpeakers, "extend_list")
    def test_list_bgp_speaker_pagination(self, mock_extend_list):
        # List all BGP Speakers with pagination support.
        cmd = bgp_speaker.ListSpeakers(test_cli20.MyApp(sys.stdout),
                                       None)
        self._test_list_resources_with_pagination("bgp_speakers",
                                                  cmd)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    def test_list_bgp_speaker_sort(self):
        # sorted list: bgp-speaker-list --sort-key name --sort-key id
        #                               --sort-key asc --sort-key desc
        resources = "bgp_speakers"
        cmd = bgp_speaker.ListSpeakers(test_cli20.MyApp(sys.stdout),
                                       None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_bgp_speaker_limit(self):
        # size (1000) limited list: bgp-speaker-list -P.
        resources = "bgp_speakers"
        cmd = bgp_speaker.ListSpeakers(test_cli20.MyApp(sys.stdout),
                                       None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_bgp_speaker(self):
        # Show BGP Speaker: --fields id --fields name myid.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.ShowSpeaker(test_cli20.MyApp(sys.stdout),
                                      None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])

    def test_delete_bgp_speaker(self):
        # Delete BGP Speaker: bgp_speaker_id.
        resource = 'bgp_speaker'
        cmd = bgp_speaker.DeleteSpeaker(test_cli20.MyApp(sys.stdout),
                                        None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def _test_add_remove_peer(self, action, cmd, args):
        """Add or Remove BGP Peer to/from a BGP Speaker."""
        resource = 'bgp_speaker'
        subcmd = '%s_bgp_peer' % action
        body = {'bgp_peer_id': 'peerid'}
        if action == 'add':
            retval = {'bgp_peer': 'peerid'}
            retval = self.client.serialize(retval)
            expected_code = 200
        else:
            retval = None
            expected_code = 204
        self._test_update_resource_action(resource, cmd, 'myid',
                                          subcmd, args, body, expected_code,
                                          retval)

    def test_add_peer_to_bgp_speaker(self):
        # Add peer to BGP speaker: myid peer_id=peerid
        cmd = bgp_speaker.AddPeerToSpeaker(test_cli20.MyApp(sys.stdout),
                                           None)
        args = ['myid', 'peerid']
        self._test_add_remove_peer('add', cmd, args)

    def test_remove_peer_from_bgp_speaker(self):
        # Remove peer from BGP speaker: myid peer_id=peerid
        cmd = bgp_speaker.RemovePeerFromSpeaker(test_cli20.MyApp(sys.stdout),
                                                None)
        args = ['myid', 'peerid']
        self._test_add_remove_peer('remove', cmd, args)

    def _test_add_remove_network(self, action, cmd, args):
        # Add or Remove network to/from a BGP Speaker.
        resource = 'bgp_speaker'
        subcmd = '%s_gateway_network' % action
        body = {'network_id': 'netid'}
        if action == 'add':
            retval = {'network': 'netid'}
            retval = self.client.serialize(retval)
            expected_code = 200
        else:
            retval = None
            expected_code = 204
        self._test_update_resource_action(resource, cmd, 'myid',
                                          subcmd, args, body, expected_code,
                                          retval)

    def test_add_network_to_bgp_speaker(self):
        # Add peer to BGP speaker: myid network_id=netid
        cmd = bgp_speaker.AddNetworkToSpeaker(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['myid', 'netid']
        self._test_add_remove_network('add', cmd, args)

    def test_remove_network_from_bgp_speaker(self):
        # Remove network from BGP speaker: myid network_id=netid
        cmd = bgp_speaker.RemoveNetworkFromSpeaker(
            test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'netid']
        self._test_add_remove_network('remove', cmd, args)

    def test_list_routes_advertised_by_a_bgp_speaker(self):
        # Retrieve advertised route list
        resources = 'advertised_routes'
        cmd = bgp_speaker.ListRoutesAdvertisedBySpeaker(
            test_cli20.MyApp(sys.stdout), None)
        bs_id = 'bgp_speaker_id1'
        path = ((self.client.bgp_speaker_path + '/get_advertised_routes') %
                bs_id)
        self._test_list_resources(resources, cmd, base_args=[bs_id],
                                  path=path)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils as nc_utils
from neutronclient.osc import utils as nc_osc_utils
from neutronclient.osc.v2.dynamic_routing import constants


def _get_attrs(client_manager, parsed_args):
    attrs = {}

    # Validate password
    if 'auth_type' in parsed_args:
        if parsed_args.auth_type != 'none':
            if 'password' not in parsed_args or parsed_args.password is None:
                raise exceptions.CommandError(_('Must provide password if '
                                                'auth-type is specified.'))
        if (
            parsed_args.auth_type == 'none' and
            parsed_args.password is not None
        ):
            raise exceptions.CommandError(_('Must provide auth-type if '
                                            'password is specified.'))
        attrs['auth_type'] = parsed_args.auth_type

    if parsed_args.name is not None:
        attrs['name'] = parsed_args.name
    if 'remote_as' in parsed_args:
        attrs['remote_as'] = parsed_args.remote_as
    if 'peer_ip' in parsed_args:
        attrs['peer_ip'] = parsed_args.peer_ip
    if 'password' in parsed_args:
        attrs['password'] = parsed_args.password

    if 'project' in parsed_args and parsed_args.project is not None:
        identity_client = client_manager.identity
        project_id = nc_osc_utils.find_project(
            identity_client,
            parsed_args.project,
            parsed_args.project_domain,
        ).id
        attrs['tenant_id'] = project_id
    return attrs


class CreateBgpPeer(command.ShowOne):
    _description = _("Create a BGP peer")

    def get_parser(self, prog_name):
        parser = super(CreateBgpPeer, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Name of the BGP peer to create"))
        parser.add_argument(
            '--peer-ip',
            metavar='<peer-ip-address>',
            required=True,
            help=_("Peer IP address"))
        parser.add_argument(
            '--remote-as',
            required=True,
            metavar='<peer-remote-as>',
            help=_("Peer AS number. (Integer in [%(min_val)s, %(max_val)s] "
                   "is allowed)") % {'min_val': constants.MIN_AS_NUM,
                                     'max_val': constants.MAX_AS_NUM})
        parser.add_argument(
            '--auth-type',
            metavar='<peer-auth-type>',
            choices=['none', 'md5'],
            type=nc_utils.convert_to_lowercase,
            default='none',
            help=_("Authentication algorithm. Supported algorithms: "
                   "none (default), md5"))
        parser.add_argument(
            '--password',
            metavar='<auth-password>',
            help=_("Authentication password"))
        nc_osc_utils.add_project_owner_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_attrs(self.app.client_manager, parsed_args)
        body = {constants.BGP_PEER: attrs}
        obj = client.create_bgp_peer(body)[constants.BGP_PEER]
        columns, display_columns = column_util.get_columns(obj)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class DeleteBgpPeer(command.Command):
    _description = _("Delete a BGP peer")

    def get_parser(self, prog_name):
        parser = super(DeleteBgpPeer, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_peer',
            metavar="<bgp-peer>",
            help=_("BGP peer to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGP_PEER,
                                  parsed_args.bgp_peer)['id']
        client.delete_bgp_peer(id)


class ListBgpPeer(command.Lister):
    _description = _("List BGP peers")

    def take_action(self, parsed_args):
        data = self.app.client_manager.neutronclient.list_bgp_peers()
        headers = ('ID', 'Name', 'Peer IP', 'Remote AS')
        columns = ('id', 'name', 'peer_ip', 'remote_as')
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[constants.BGP_PEERS]))


class SetBgpPeer(command.Command):
    _description = _("Update a BGP peer")
    resource = constants.BGP_PEER

    def get_parser(self, prog_name):
        parser = super(SetBgpPeer, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            help=_("Updated name of the BGP peer"))
        parser.add_argument(
            '--password',
            metavar='<auth-password>',
            help=_("Updated authentication password"))
        parser.add_argument(
            'bgp_peer',
            metavar="<bgp-peer>",
            help=_("BGP peer to update (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGP_PEER,
                                  parsed_args.bgp_peer)['id']
        attrs = _get_attrs(self.app.client_manager, parsed_args)
        body = {}
        body[constants.BGP_PEER] = attrs
        client.update_bgp_peer(id, body)


class ShowBgpPeer(command.ShowOne):
    _description = _("Show information for a BGP peer")

    def get_parser(self, prog_name):
        parser = super(ShowBgpPeer, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_peer',
            metavar="<bgp-peer>",
            help=_("BGP peer to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGP_PEER,
                                  parsed_args.bgp_peer)['id']
        obj = client.show_bgp_peer(id)[constants.BGP_PEER]
        columns, display_columns = column_util.get_columns(obj)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data



# Copyright 2016 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import copy
import re
from unittest import mock

from osc_lib import exceptions
from osc_lib.tests import utils

from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.fwaas import constants as const
from neutronclient.osc.v2.fwaas import firewallpolicy
from neutronclient.tests.unit.osc.v2 import fakes as test_fakes
from neutronclient.tests.unit.osc.v2.fwaas import common
from neutronclient.tests.unit.osc.v2.fwaas import fakes


_fwp = fakes.FirewallPolicy().create()
CONVERT_MAP = {
    'share': 'shared',
    'no_share': 'shared',
    'project': 'tenant_id',
    'port': 'ports',
    'name': 'name',
    'id': 'id',
    'firewall_rule': 'firewall_rules',
    'description': 'description'
}


def _generate_data(ordered_dict=None, data=None):
    source = ordered_dict if ordered_dict else _fwp
    if data:
        source.update(data)
    return tuple(source[key] for key in source)


def _generate_req_and_res(verifylist):
    request = dict(verifylist)
    response = copy.deepcopy(_fwp)
    for key, val in verifylist:
        converted = CONVERT_MAP.get(key, key)
        del request[key]
        if re.match('^no_', key) and val is True:
            new_value = None
        elif key == 'enable' and val:
            new_value = True
        elif key == 'disable' and val:
            new_value = False
        else:
            new_value = val
        request[converted] = new_value
        response[converted] = new_value
    return request, response


class TestFirewallPolicy(test_fakes.TestNeutronClientOSCV2):

    def check_results(self, headers, data, exp_req, is_list=False):
        if is_list:
            req_body = {self.res_plural: [exp_req]}
        else:
            req_body = {self.res: exp_req}
        self.mocked.assert_called_once_with(req_body)
        self.assertEqual(self.ordered_headers, headers)
        self.assertEqual(self.ordered_data, data)

    def setUp(self):
        super(TestFirewallPolicy, self).setUp()

        def _find_resource(*args, **kwargs):
            rule_id = args[1]
            rules = []
            if self.res in args[0]:
                rules = _fwp['firewall_rules']
            return {'id': rule_id, 'firewall_rules': rules}

        self.neutronclient.find_resource = mock.Mock(
            side_effect=_find_resource)
        osc_utils.find_project = mock.Mock()
        osc_utils.find_project.id = _fwp['tenant_id']
        self.res = 'firewall_policy'
        self.res_plural = 'firewall_policies'
        self.resource = _fwp
        self.list_headers = (
            'ID',
            'Name',
            'Firewall Rules',
        )
        self.list_data = (
            _fwp['id'],
            _fwp['name'],
            _fwp['firewall_rules'],
        )
        self.headers = tuple(self.list_headers + (
            'Description',
            'Audited',
            'Shared',
            'Project')
        )
        self.data = _generate_data()
        self.ordered_headers = (
            'Audited',
            'Description',
            'Firewall Rules',
            'ID',
            'Name',
            'Project',
            'Shared',
        )
        self.ordered_data = (
            _fwp['audited'],
            _fwp['description'],
            _fwp['firewall_rules'],
            _fwp['id'],
            _fwp['name'],
            _fwp['tenant_id'],
            _fwp['shared'],
        )
        self.ordered_columns = (
            'audited',
            'description',
            'firewall_rules',
            'id',
            'name',
            'tenant_id',
            'shared',
        )


class TestCreateFirewallPolicy(TestFirewallPolicy, common.TestCreateFWaaS):

    def setUp(self):
        super(TestCreateFirewallPolicy, self).setUp()
        self.neutronclient.create_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.create_fwaas_firewall_policy
        self.cmd = firewallpolicy.CreateFirewallPolicy(self.app,
                                                       self.namespace)

    def _update_expect_response(self, request, response):
        """Set expected request and response

        :param request
            A dictionary of request body(dict of verifylist)
        :param response
            A OrderedDict of request body
        """
        # Update response body
        self.neutronclient.create_fwaas_firewall_policy.return_value = \
            {self.res: dict(response)}
        osc_utils.find_project.return_value.id = response['tenant_id']
        # Update response(finally returns 'data')
        self.data = _generate_data(data=response)
        self.ordered_data = tuple(
            response[column] for column in self.ordered_columns
        )

    def test_create_with_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_mandatory_param(self):
        name = 'my-fwg'
        arglist = [
            name,
        ]
        verifylist = [
            ('name', name),
        ]
        request, response = _generate_req_and_res(verifylist)
        self._update_expect_response(request, response)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.check_results(headers, data, request)

    def test_create_with_rules(self):
        name = 'my-fwg'
        rule1 = 'rule1'
        rule2 = 'rule2'

        def _mock_policy(*args, **kwargs):
            self.neutronclient.find_resource.assert_called_with(
                'firewall_rule', args[1], cmd_resource=const.CMD_FWR)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = _mock_policy

        arglist = [
            name,
            '--firewall-rule', rule1,
            '--firewall-rule', rule2,
        ]
        verifylist = [
            ('name', name),
            ('firewall_rule', [rule1, rule2]),
        ]
        request, response = _generate_req_and_res(verifylist)
        self._update_expect_response(request, response)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)
        self.assertEqual(2, self.neutronclient.find_resource.call_count)

        self.check_results(headers, data, request)

    def test_create_with_all_params(self):
        name = 'my-fwp'
        desc = 'my-desc'
        rule1 = 'rule1'
        rule2 = 'rule2'
        project = 'my-tenant'
        arglist = [
            name,
            '--description', desc,
            '--firewall-rule', rule1,
            '--firewall-rule', rule2,
            '--project', project,
            '--share',
            '--audited',
        ]
        verifylist = [
            ('name', name),
            ('description', desc),
            ('firewall_rule', [rule1, rule2]),
            ('project', project),
            ('share', True),
            ('audited', True),
        ]
        request, response = _generate_req_and_res(verifylist)
        self._update_expect_response(request, response)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.check_results(headers, data, request)

    def test_create_with_firewall_rule_and_no(self):
        name = 'my-fwp'
        rule1 = 'rule1'
        rule2 = 'rule2'
        arglist = [
            name,
            '--firewall-rule', rule1,
            '--firewall-rule', rule2,
            '--no-firewall-rule',
        ]
        verifylist = [
            ('name', name),
            ('firewall_rule', [rule1, rule2]),
            ('no_firewall_rule', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_shared_and_no_share(self):
        name = 'my-fwp'
        arglist = [
            name,
            '--share',
            '--no-share',
        ]
        verifylist = [
            ('name', name),
            ('share', True),
            ('no_share', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_audited_and_no(self):
        name = 'my-fwp'
        arglist = [
            name,
            '--audited',
            '--no-audited',
        ]
        verifylist = [
            ('name', name),
            ('audited', True),
            ('no_audited', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestListFirewallPolicy(TestFirewallPolicy, common.TestListFWaaS):

    def setUp(self):
        super(TestListFirewallPolicy, self).setUp()
        self.neutronclient.list_fwaas_firewall_policies = mock.Mock(
            return_value={'firewall_policies': [_fwp]})
        self.mocked = self.neutronclient.list_fwaas_firewall_policies
        self.cmd = firewallpolicy.ListFirewallPolicy(self.app, self.namespace)


class TestShowFirewallPolicy(TestFirewallPolicy, common.TestShowFWaaS):

    def setUp(self):
        super(TestShowFirewallPolicy, self).setUp()
        self.neutronclient.show_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.show_fwaas_firewall_policy
        self.cmd = firewallpolicy.ShowFirewallPolicy(self.app, self.namespace)


class TestSetFirewallPolicy(TestFirewallPolicy, common.TestSetFWaaS):

    def setUp(self):
        super(TestSetFirewallPolicy, self).setUp()
        self.neutronclient.update_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.update_fwaas_firewall_policy
        self.cmd = firewallpolicy.SetFirewallPolicy(self.app, self.namespace)

    def test_set_rules(self):
        target = self.resource['id']
        rule1 = 'new_rule1'
        rule2 = 'new_rule2'

        def _mock_policy(*args, **kwargs):
            # 1. Find specified firewall_policy
            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource=const.CMD_FWP)
            # 2. Find specified firewall_policy's 'firewall_rules' attribute
            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, args[1], cmd_resource=const.CMD_FWP)
                return {'firewall_rules': _fwp['firewall_rules']}
            # 3. Find specified firewall_rule
            if self.neutronclient.find_resource.call_count == 3:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', args[1], cmd_resource=const.CMD_FWR)
            # 4. Find specified firewall_rule
            if self.neutronclient.find_resource.call_count == 4:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', args[1], cmd_resource=const.CMD_FWR)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = _mock_policy

        arglist = [
            target,
            '--firewall-rule', rule1,
            '--firewall-rule', rule2,
        ]
        verifylist = [
            (self.res, target),
            ('firewall_rule', [rule1, rule2]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        expect = _fwp['firewall_rules'] + [rule1, rule2]
        body = {self.res: {'firewall_rules': expect}}
        self.mocked.assert_called_once_with(target, body)
        self.assertEqual(4, self.neutronclient.find_resource.call_count)
        self.assertIsNone(result)

    def test_set_no_rules(self):
        target = self.resource['id']
        arglist = [target, '--no-firewall-rule']
        verifylist = [
            (self.res, target),
            ('no_firewall_rule', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        body = {self.res: {'firewall_rules': []}}
        self.mocked.assert_called_once_with(target, body)
        self.assertIsNone(result)

    def test_set_rules_and_no_rules(self):
        target = self.resource['id']
        rule1 = 'rule1'
        arglist = [
            target,
            '--firewall-rule', rule1,
            '--no-firewall-rule',
        ]
        verifylist = [
            (self.res, target),
            ('firewall_rule', [rule1]),
            ('no_firewall_rule', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        body = {self.res: {'firewall_rules': [rule1]}}
        self.mocked.assert_called_once_with(target, body)
        self.assertEqual(2, self.neutronclient.find_resource.call_count)
        self.assertIsNone(result)

    def test_set_audited(self):
        target = self.resource['id']
        arglist = [target, '--audited']
        verifylist = [
            (self.res, target),
            ('audited', True),
        ]
        body = {'audited': True}
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target, {self.res: body})
        self.assertIsNone(result)

    def test_set_no_audited(self):
        target = self.resource['id']
        arglist = [target, '--no-audited']
        verifylist = [
            (self.res, target),
            ('no_audited', True),
        ]
        body = {'audited': False}
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target, {self.res: body})
        self.assertIsNone(result)

    def test_set_audited_and_no_audited(self):
        target = self.resource['id']
        arglist = [
            target,
            '--audited',
            '--no-audited',
        ]
        verifylist = [
            (self.res, target),
            ('audited', True),
            ('no_audited', True),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_set_and_raises(self):
        self.neutronclient.update_fwaas_firewall_policy = mock.Mock(
            side_effect=Exception)
        target = self.resource['id']
        arglist = [target, '--name', 'my-name']
        verifylist = [(self.res, target), ('name', 'my-name')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestDeleteFirewallPolicy(TestFirewallPolicy, common.TestDeleteFWaaS):

    def setUp(self):
        super(TestDeleteFirewallPolicy, self).setUp()
        self.neutronclient.delete_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.delete_fwaas_firewall_policy
        self.cmd = firewallpolicy.DeleteFirewallPolicy(
            self.app, self.namespace)


class TestFirewallPolicyInsertRule(TestFirewallPolicy):

    def setUp(self):
        super(TestFirewallPolicyInsertRule, self).setUp()
        self.neutronclient.insert_rule_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.insert_rule_fwaas_firewall_policy
        self.cmd = firewallpolicy.FirewallPolicyInsertRule(self.app,
                                                           self.namespace)

    def test_insert_firewall_rule(self):
        target = self.resource['id']
        rule = 'new-rule'
        before = 'before'
        after = 'after'

        def _mock_policy(*args, **kwargs):
            # 1. Find specified firewall_policy
            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource=const.CMD_FWP)
            # 2. Find specified firewall_rule
            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', args[1], cmd_resource=const.CMD_FWR)
            # 3. Find specified firewall_rule as 'before'
            if self.neutronclient.find_resource.call_count == 3:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', args[1], cmd_resource=const.CMD_FWR)
            # 4. Find specified firewall_rule as 'after'
            if self.neutronclient.find_resource.call_count == 4:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', args[1], cmd_resource=const.CMD_FWR)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = _mock_policy

        arglist = [
            target,
            rule,
            '--insert-before', before,
            '--insert-after', after,
        ]
        verifylist = [
            (self.res, target),
            ('firewall_rule', rule),
            ('insert_before', before),
            ('insert_after', after),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {
                'firewall_rule_id': rule,
                'insert_before': before,
                'insert_after': after
            })
        self.assertIsNone(result)
        self.assertEqual(4, self.neutronclient.find_resource.call_count)

    def test_insert_with_no_firewall_rule(self):
        target = self.resource['id']
        arglist = [
            target,
        ]
        verifylist = [
            (self.res, target),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestFirewallPolicyRemoveRule(TestFirewallPolicy):

    def setUp(self):
        super(TestFirewallPolicyRemoveRule, self).setUp()
        self.neutronclient.remove_rule_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.remove_rule_fwaas_firewall_policy
        self.cmd = firewallpolicy.FirewallPolicyRemoveRule(self.app,
                                                           self.namespace)

    def test_remove_firewall_rule(self):
        target = self.resource['id']
        rule = 'remove-rule'

        def _mock_policy(*args, **kwargs):
            # 1. Find specified firewall_policy
            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource=const.CMD_FWP)
            # 2. Find specified firewall_rule
            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', rule, cmd_resource=const.CMD_FWR)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = mock.Mock(
            side_effect=_mock_policy)

        arglist = [
            target,
            rule,
        ]
        verifylist = [
            (self.res, target),
            ('firewall_rule', rule),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.mocked.assert_called_once_with(
            target, {'firewall_rule_id': rule})
        self.assertIsNone(result)
        self.assertEqual(2, self.neutronclient.find_resource.call_count)

    def test_remove_with_no_firewall_rule(self):
        target = self.resource['id']
        arglist = [
            target,
        ]
        verifylist = [
            (self.res, target),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestUnsetFirewallPolicy(TestFirewallPolicy, common.TestUnsetFWaaS):

    def setUp(self):
        super(TestUnsetFirewallPolicy, self).setUp()
        self.neutronclient.update_fwaas_firewall_policy = mock.Mock(
            return_value={self.res: _fwp})
        self.mocked = self.neutronclient.update_fwaas_firewall_policy
        self.cmd = firewallpolicy.UnsetFirewallPolicy(self.app, self.namespace)

    def test_unset_audited(self):
        target = self.resource['id']
        arglist = [
            target,
            '--audited',
        ]
        verifylist = [
            (self.res, target),
            ('audited', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        body = {self.res: {'audited': False}}
        self.mocked.assert_called_once_with(target, body)
        self.assertIsNone(result)

    def test_unset_firewall_rule_not_matched(self):
        _fwp['firewall_rules'] = ['old_rule']
        target = self.resource['id']
        rule = 'new_rule'
        arglist = [
            target,
            '--firewall-rule', rule,
        ]
        verifylist = [
            (self.res, target),
            ('firewall_rule', [rule]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        body = {self.res: {'firewall_rules': _fwp['firewall_rules']}}
        self.mocked.assert_called_once_with(target, body)
        self.assertIsNone(result)

    def test_unset_firewall_rule_matched(self):
        _fwp['firewall_rules'] = ['rule1', 'rule2']
        target = self.resource['id']
        rule = 'rule1'

        def _mock_policy(*args, **kwargs):
            # 1. Find specified firewall_policy
            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource=const.CMD_FWP)
            # 2. Find 'firewall_rules' attribute from specified firewall_policy
            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource=const.CMD_FWP)
                return {'firewall_rules': _fwp['firewall_rules']}
            # 3. Find specified 'firewall_rule'
            if self.neutronclient.find_resource.call_count == 3:
                self.neutronclient.find_resource.assert_called_with(
                    'firewall_rule', rule, cmd_resource=const.CMD_FWR)
            return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = _mock_policy

        arglist = [
            target,
            '--firewall-rule', rule,
        ]
        verifylist = [
            (self.res, target),
            ('firewall_rule', [rule]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        body = {self.res: {'firewall_rules': ['rule2']}}
        self.mocked.assert_called_once_with(target, body)
        self.assertIsNone(result)
        self.assertEqual(3, self.neutronclient.find_resource.call_count)

    def test_unset_all_firewall_rule(self):
        target = self.resource['id']
        arglist = [
            target,
            '--all-firewall-rule',
        ]
        verifylist = [
            (self.res, target),
            ('all_firewall_rule', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        body = {self.res: {'firewall_rules': []}}
        self.mocked.assert_called_once_with(target, body)
        self.assertIsNone(result)



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


class ListVip(neutronV20.ListCommand):
    """List vips that belong to a given tenant."""

    resource = 'vip'
    list_columns = ['id', 'name', 'algorithm', 'address', 'protocol',
                    'admin_state_up', 'status']
    pagination_support = True
    sorting_support = True


class ShowVip(neutronV20.ShowCommand):
    """Show information of a given vip."""

    resource = 'vip'


class CreateVip(neutronV20.CreateCommand):
    """Create a vip."""

    resource = 'vip'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'pool_id', metavar='POOL',
            help=_('ID or name of the pool to which this vip belongs.'))
        parser.add_argument(
            '--address',
            help=_('IP address of the vip.'))
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--connection-limit',
            help=_('The maximum number of connections per second allowed for '
                   'the vip. Valid values: a positive integer or -1 '
                   'for unlimited (default).'))
        parser.add_argument(
            '--description',
            help=_('Description of the vip to be created.'))
        parser.add_argument(
            '--name',
            required=True,
            help=_('Name of the vip to be created.'))
        parser.add_argument(
            '--protocol-port',
            required=True,
            help=_('TCP port on which to listen for client traffic that is '
                   'associated with the vip address.'))
        parser.add_argument(
            '--protocol',
            required=True, choices=['TCP', 'HTTP', 'HTTPS'],
            help=_('Protocol for balancing.'))
        parser.add_argument(
            '--subnet-id', metavar='SUBNET',
            required=True,
            help=_('The subnet on which to allocate the vip address.'))

    def args2body(self, parsed_args):
        _pool_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'pool', parsed_args.pool_id)
        _subnet_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'subnet', parsed_args.subnet_id)

        body = {'pool_id': _pool_id,
                'admin_state_up': parsed_args.admin_state,
                'subnet_id': _subnet_id}
        neutronV20.update_dict(parsed_args, body,
                               ['address', 'connection_limit', 'description',
                                'name', 'protocol_port', 'protocol',
                                'tenant_id'])
        return {self.resource: body}


class UpdateVip(neutronV20.UpdateCommand):
    """Update a given vip."""

    resource = 'vip'


class DeleteVip(neutronV20.DeleteCommand):
    """Delete a given vip."""

    resource = 'vip'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneauth1 import session
from oslo_utils import uuidutils
from tempest.lib import base
import testtools

from neutronclient.common import exceptions
from neutronclient.tests.functional import base as func_base
from neutronclient.v2_0 import client as v2_client


class LibraryTestCase(base.BaseTestCase):

    def setUp(self):
        super(LibraryTestCase, self).setUp()
        self.client = self._get_client()

    def _get_client(self):
        cloud_config = func_base.get_cloud_config()
        keystone_auth = cloud_config.get_auth()
        (verify, cert) = cloud_config.get_requests_verify_args()

        ks_session = session.Session(
            auth=keystone_auth,
            verify=verify,
            cert=cert)
        return v2_client.Client(session=ks_session)

    def test_list_network(self):
        nets = self.client.list_networks()
        self.assertIsInstance(nets['networks'], list)

    def test_post_put_delete_network(self):
        name = uuidutils.generate_uuid()
        net = self.client.create_network({'network': {'name': name}})
        net_id = net['network']['id']
        self.assertEqual(name, net['network']['name'])
        name2 = uuidutils.generate_uuid()
        net = self.client.update_network(net_id, {'network': {'name': name2}})
        self.assertEqual(name2, net['network']['name'])
        self.client.delete_network(net_id)
        with testtools.ExpectedException(exceptions.NetworkNotFoundClient):
            self.client.show_network(net_id)

    def test_get_auth_ref(self):
        # Call some API call to ensure the client is authenticated.
        self.client.list_networks()
        auth_ref = self.client.httpclient.get_auth_ref()
        self.assertIsNotNone(auth_ref)
        self.assertIsNotNone(auth_ref.role_names)



# Copyright (C) 2013 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from oslo_utils import uuidutils
import osprofiler.profiler
import osprofiler.web
from requests_mock.contrib import fixture as mock_fixture
import testtools

from neutronclient import client
from neutronclient.common import exceptions


AUTH_TOKEN = 'test_token'
END_URL = 'test_url'
METHOD = 'GET'
URL = 'http://test.test:1234/v2.0/test'
BODY = 'IAMFAKE'


class TestHTTPClientMixin(object, metaclass=abc.ABCMeta):

    def setUp(self):
        super(TestHTTPClientMixin, self).setUp()

        self.requests = self.useFixture(mock_fixture.Fixture())
        self.http = self.initialize()

    @abc.abstractmethod
    def initialize(self):
        """Return client class, instance."""

    def _test_headers(self, expected_headers, **kwargs):
        # Test headers.
        self.requests.register_uri(METHOD, URL,
                                   request_headers=expected_headers)
        self.http.request(URL, METHOD, **kwargs)
        self.assertEqual(kwargs.get('body'), self.requests.last_request.body)

    def test_headers_without_body(self):
        self._test_headers({'Accept': 'application/json'})

    def test_headers_with_body(self):
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/json'}
        self._test_headers(headers, body=BODY)

    def test_headers_without_body_with_content_type(self):
        headers = {'Accept': 'application/json'}
        self._test_headers(headers, content_type='application/json')

    def test_headers_with_body_with_content_type(self):
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/json'}
        self._test_headers(headers, body=BODY, content_type='application/json')

    def test_headers_defined_in_headers(self):
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/json'}
        self._test_headers(headers, body=BODY, headers=headers)

    def test_osprofiler_headers_are_injected(self):
        osprofiler.profiler.init('SWORDFISH')
        self.addCleanup(osprofiler.profiler.clean)

        headers = {'Accept': 'application/json'}
        headers.update(osprofiler.web.get_trace_id_headers())
        self._test_headers(headers)


class TestHTTPClient(TestHTTPClientMixin, testtools.TestCase):

    def initialize(self):
        return client.HTTPClient(token=AUTH_TOKEN, endpoint_url=END_URL)

    def test_request_error(self):
        def cb(*args, **kwargs):
            raise Exception('error msg')

        self.requests.get(URL, body=cb)
        self.assertRaises(
            exceptions.ConnectionFailed,
            self.http._cs_request,
            URL, METHOD
        )

    def test_request_success(self):
        text = 'test content'
        self.requests.register_uri(METHOD, URL, text=text)

        resp, resp_text = self.http._cs_request(URL, METHOD)
        self.assertEqual(200, resp.status_code)
        self.assertEqual(text, resp_text)

    def test_request_unauthorized(self):
        text = 'unauthorized message'
        self.requests.register_uri(METHOD, URL, status_code=401, text=text)
        e = self.assertRaises(exceptions.Unauthorized,
                              self.http._cs_request, URL, METHOD)
        self.assertEqual(text, e.message)

    def test_request_forbidden_is_returned_to_caller(self):
        text = 'forbidden message'
        self.requests.register_uri(METHOD, URL, status_code=403, text=text)

        resp, resp_text = self.http._cs_request(URL, METHOD)
        self.assertEqual(403, resp.status_code)
        self.assertEqual(text, resp_text)

    def test_do_request_success(self):
        text = 'test content'
        self.requests.register_uri(METHOD, END_URL + URL, text=text)

        resp, resp_text = self.http.do_request(URL, METHOD)
        self.assertEqual(200, resp.status_code)
        self.assertEqual(text, resp_text)

    def test_do_request_with_headers_success(self):
        text = 'test content'
        self.requests.register_uri(METHOD, END_URL + URL, text=text,
                                   request_headers={'key': 'value'})

        resp, resp_text = self.http.do_request(URL, METHOD,
                                               headers={'key': 'value'})
        self.assertEqual(200, resp.status_code)
        self.assertEqual(text, resp_text)


class TestHTTPClientWithReqId(TestHTTPClientMixin, testtools.TestCase):
    """Tests for when global_request_id is set."""

    def initialize(self):
        self.req_id = "req-%s" % uuidutils.generate_uuid()
        return client.HTTPClient(token=AUTH_TOKEN, endpoint_url=END_URL,
                                 global_request_id=self.req_id)

    def test_request_success(self):
        headers = {
            'Accept': 'application/json',
            'X-OpenStack-Request-ID': self.req_id
        }
        self.requests.register_uri(METHOD, URL, request_headers=headers)
        self.http.request(URL, METHOD)



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.neutron.v2_0.lb import healthmonitor
from neutronclient.tests.unit import test_cli20


class CLITestV20LbHealthmonitorJSON(test_cli20.CLITestV20Base):
    def test_create_healthmonitor_with_mandatory_params(self):
        # lb-healthmonitor-create with mandatory params only.
        resource = 'health_monitor'
        cmd = healthmonitor.CreateHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        admin_state_up = False
        delay = '60'
        max_retries = '2'
        timeout = '10'
        type = 'TCP'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--admin-state-down',
                '--delay', delay,
                '--max-retries', max_retries,
                '--timeout', timeout,
                '--type', type,
                '--tenant-id', tenant_id]
        position_names = ['admin_state_up', 'delay', 'max_retries', 'timeout',
                          'type', 'tenant_id']
        position_values = [admin_state_up, delay, max_retries, timeout, type,
                           tenant_id]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values)

    def test_create_healthmonitor_with_all_params(self):
        # lb-healthmonitor-create with all params set.
        resource = 'health_monitor'
        cmd = healthmonitor.CreateHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        admin_state_up = False
        delay = '60'
        expected_codes = '200-202,204'
        http_method = 'HEAD'
        max_retries = '2'
        timeout = '10'
        type = 'TCP'
        tenant_id = 'my-tenant'
        url_path = '/health'
        my_id = 'my-id'
        args = ['--admin-state-down',
                '--delay', delay,
                '--expected-codes', expected_codes,
                '--http-method', http_method,
                '--max-retries', max_retries,
                '--timeout', timeout,
                '--type', type,
                '--tenant-id', tenant_id,
                '--url-path', url_path]
        position_names = ['admin_state_up', 'delay',
                          'expected_codes', 'http_method',
                          'max_retries', 'timeout',
                          'type', 'tenant_id', 'url_path']
        position_values = [admin_state_up, delay,
                           expected_codes, http_method,
                           max_retries, timeout,
                           type, tenant_id, url_path]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values)

    def test_list_healthmonitors(self):
        # lb-healthmonitor-list.
        resources = "health_monitors"
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, True)

    def test_list_healthmonitors_pagination(self):
        # lb-healthmonitor-list.
        resources = "health_monitors"
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_healthmonitors_sort(self):
        # lb-healthmonitor-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "health_monitors"
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_healthmonitors_limit(self):
        # lb-healthmonitor-list -P.
        resources = "health_monitors"
        cmd = healthmonitor.ListHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_healthmonitor_id(self):
        # lb-healthmonitor-show test_id.
        resource = 'health_monitor'
        cmd = healthmonitor.ShowHealthMonitor(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_update_health_monitor(self):
        # lb-healthmonitor-update  myid --name myname --tags a b.
        resource = 'health_monitor'
        cmd = healthmonitor.UpdateHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--timeout', '5'],
                                   {'timeout': '5', })

    def test_delete_healthmonitor(self):
        # lb-healthmonitor-delete my-id."""
        resource = 'health_monitor'
        cmd = healthmonitor.DeleteHealthMonitor(test_cli20.MyApp(sys.stdout),
                                                None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_associate_healthmonitor(self):
        cmd = healthmonitor.AssociateHealthMonitor(
            test_cli20.MyApp(sys.stdout),
            None)
        resource = 'health_monitor'
        health_monitor_id = 'hm-id'
        pool_id = 'p_id'
        args = [health_monitor_id, pool_id]

        body = {resource: {'id': health_monitor_id}}
        result = {resource: {'id': health_monitor_id}, }
        result_str = self.client.serialize(result)

        path = getattr(self.client,
                       "associate_pool_health_monitors_path") % pool_id
        return_tup = (test_cli20.MyResp(200), result_str)
        cmd_parser = cmd.get_parser('test_' + resource)
        parsed_args = cmd_parser.parse_args(args)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.end_url(path), 'POST',
            body=test_cli20.MyComparator(body, self.client),
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def test_disassociate_healthmonitor(self):
        cmd = healthmonitor.DisassociateHealthMonitor(
            test_cli20.MyApp(sys.stdout),
            None)
        resource = 'health_monitor'
        health_monitor_id = 'hm-id'
        pool_id = 'p_id'
        args = [health_monitor_id, pool_id]

        path = (getattr(self.client,
                        "disassociate_pool_health_monitors_path") %
                {'pool': pool_id, 'health_monitor': health_monitor_id})
        return_tup = (test_cli20.MyResp(204), None)
        cmd_parser = cmd.get_parser('test_' + resource)
        parsed_args = cmd_parser.parse_args(args)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.end_url(path), 'DELETE',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))



# Copyright 2017-2018 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import copy
from unittest import mock

from osc_lib import exceptions
from osc_lib.tests import utils
import testtools

from neutronclient.osc import utils as osc_utils
from neutronclient.osc.v2.logging import network_log
from neutronclient.tests.unit.osc.v2 import fakes as test_fakes
from neutronclient.tests.unit.osc.v2.logging import fakes


_log = fakes.NetworkLog().create()
RES_TYPE_SG = 'security_group'
RES_TYPE_FWG = 'firewall_group'
CONVERT_MAP = {
    'project': 'project_id',
    'enable': 'enabled',
    'disable': 'enabled',
    'target': 'target_id',
    'resource': 'resource_id',
    'event': 'event',
}


def _generate_data(ordered_dict=None, data=None):
    source = ordered_dict if ordered_dict else _log
    if data:
        source.update(data)
    return tuple(source[key] for key in source)


def _generate_req_and_res(verifylist):
    request = dict(verifylist)
    response = copy.deepcopy(_log)
    for key, val in verifylist:
        converted = CONVERT_MAP.get(key, key)
        del request[key]
        if key == 'enable' and val:
            new_value = True
        elif key == 'disable' and val:
            new_value = False
        else:
            new_value = val
        request[converted] = new_value
        response[converted] = new_value
    return request, response


class TestNetworkLog(test_fakes.TestNeutronClientOSCV2):

    def check_results(self, headers, data, exp_req, is_list=False):
        if is_list:
            req_body = {'logs': [exp_req]}
        else:
            req_body = {'log': exp_req}
        self.mocked.assert_called_once_with(req_body)
        self.assertEqual(self.ordered_headers, headers)
        self.assertEqual(self.ordered_data, data)

    def setUp(self):
        super(TestNetworkLog, self).setUp()
        self.neutronclient.find_resource = mock.Mock()
        self.neutronclient.find_resource.side_effect = \
            lambda x, y, **k: {'id': y}
        osc_utils.find_project = mock.Mock()
        osc_utils.find_project.id = _log['project_id']
        self.res = _log
        self.headers = (
            'ID',
            'Description',
            'Enabled',
            'Name',
            'Target',
            'Project',
            'Resource',
            'Type',
            'Event',
        )
        self.data = _generate_data()
        self.ordered_headers = (
            'Description',
            'Enabled',
            'Event',
            'ID',
            'Name',
            'Project',
            'Resource',
            'Target',
            'Type',
        )
        self.ordered_data = (
            _log['description'],
            _log['enabled'],
            _log['event'],
            _log['id'],
            _log['name'],
            _log['project_id'],
            _log['resource_id'],
            _log['target_id'],
            _log['resource_type'],
        )
        self.ordered_columns = (
            'description',
            'enabled',
            'event',
            'id',
            'name',
            'project_id',
            'resource_id',
            'target_id',
            'resource_type',
        )


class TestCreateNetworkLog(TestNetworkLog):

    def setUp(self):
        super(TestCreateNetworkLog, self).setUp()
        self.neutronclient.create_network_log = mock.Mock(
            return_value={'log': _log})
        self.mocked = self.neutronclient.create_network_log
        self.cmd = network_log.CreateNetworkLog(self.app, self.namespace)
        loggables = {
            "loggable_resources": [{"type": RES_TYPE_SG},
                                   {"type": RES_TYPE_FWG}]
        }
        self.neutronclient.list_network_loggable_resources = mock.Mock(
            return_value=loggables)

    def _update_expect_response(self, request, response):
        """Set expected request and response

        :param request
            A dictionary of request body(dict of verifylist)
        :param response
            A OrderedDict of request body
        """
        # Update response body
        self.neutronclient.create_network_log.return_value = \
            {'log': dict(response)}
        osc_utils.find_project.return_value.id = response['project_id']
        # Update response(finally returns 'data')
        self.data = _generate_data(ordered_dict=response)
        self.ordered_data = tuple(
            response[column] for column in self.ordered_columns
        )

    def _set_all_params(self, args={}):
        name = args.get('name', 'my-log')
        desc = args.get('description', 'my-description-for-log')
        event = args.get('event', 'ACCEPT')
        resource = args.get('resource', 'id-target-log')
        target = args.get('target', 'id-target-log')
        resource_type = args.get('resource_type', 'security_group')
        project = args.get('project_id', 'id-my-project')

        arglist = [
            name,
            '--description', desc,
            '--enable',
            '--target', target,
            '--resource', resource,
            '--event', event,
            '--project', project,
            '--resource-type', resource_type,
        ]
        verifylist = [
            ('description', desc),
            ('enable', True),
            ('event', event),
            ('name', name),
            ('target', target),
            ('project', project),
            ('resource', target),
            ('resource_type', resource_type),
        ]
        return arglist, verifylist

    def _test_create_with_all_params(self, args={}):
        arglist, verifylist = self._set_all_params(args)
        request, response = _generate_req_and_res(verifylist)
        self._update_expect_response(request, response)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.check_results(headers, data, request)

    def test_create_with_no_options_and_raise(self):
        arglist = []
        verifylist = []

        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_mandatory_params(self):
        name = self.res['name']
        arglist = [
            name,
            '--resource-type', RES_TYPE_SG,
        ]
        verifylist = [
            ('name', name),
            ('resource_type', RES_TYPE_SG),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)
        expect = {
            'name': self.res['name'],
            'resource_type': self.res['resource_type'],
        }
        self.mocked.assert_called_once_with({'log': expect})
        self.assertEqual(self.ordered_headers, headers)
        self.assertEqual(self.ordered_data, data)

    def test_create_with_disable(self):
        name = self.res['name']
        arglist = [
            name,
            '--resource-type', RES_TYPE_SG,
            '--disable',
        ]
        verifylist = [
            ('name', name),
            ('resource_type', RES_TYPE_SG),
            ('disable', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)
        expect = {
            'name': self.res['name'],
            'resource_type': self.res['resource_type'],
            'enabled': False,
        }
        self.mocked.assert_called_once_with({'log': expect})
        self.assertEqual(self.ordered_headers, headers)
        self.assertEqual(self.ordered_data, data)

    def test_create_with_all_params(self):
        self._test_create_with_all_params()

    def test_create_with_all_params_event_drop(self):
        self._test_create_with_all_params({'event': 'DROP'})

    def test_create_with_all_params_event_all(self):
        self._test_create_with_all_params({'event': 'ALL'})

    def test_create_with_all_params_except_event(self):
        arglist, verifylist = self._set_all_params({'event': ''})
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_all_params_event_upper_capitalized(self):
        for event in ('all', 'All', 'dROP', 'accePt', 'accept', 'drop'):
            arglist, verifylist = self._set_all_params({'event': event})
            self.assertRaises(
                testtools.matchers._impl.MismatchError,
                self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_all_params_resource_type_upper_capitalized(self):
        for res_type in ('SECURITY_GROUP', 'Security_group', 'security_Group'):
            arglist, verifylist = self._set_all_params(
                {'resource_type': res_type})
            self.assertRaises(
                testtools.matchers._impl.MismatchError,
                self.check_parser, self.cmd, arglist, verifylist)

    def test_create_with_valid_fwg_resource(self):
        name = self.res['name']
        resource_id = 'valid_fwg_id'
        resource_type = RES_TYPE_FWG
        # Test with valid FWG ID
        with mock.patch.object(self.neutronclient, 'find_resource',
                               return_value={'id': resource_id}):
            arglist = [name,
                       '--resource-type', resource_type,
                       '--resource', resource_id
                       ]
            verifylist = [
                ('name', name),
                ('resource_type', resource_type),
                ('resource', resource_id)
            ]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            headers, data = self.cmd.take_action(parsed_args)
            expect = {
                'name': self.res['name'],
                'resource_type': RES_TYPE_FWG,
                'resource_id': 'valid_fwg_id',
            }
            self.neutronclient.find_resource.assert_called_with(
                resource_type,
                resource_id,
                cmd_resource='fwaas_firewall_group')
            self.mocked.assert_called_once_with({'log': expect})
            self.assertEqual(self.ordered_headers, headers)
            self.assertEqual(self.ordered_data, data)

    def test_create_with_invalid_fwg_resource(self):
        name = self.res['name']
        resource_id = 'invalid_fwg_id'
        resource_type = RES_TYPE_FWG
        # Test with invalid FWG ID
        with mock.patch.object(self.neutronclient, 'find_resource',
                               side_effect=exceptions.NotFound(code=0)):
            arglist = [name,
                       '--resource-type', resource_type,
                       '--resource', resource_id
                       ]
            verifylist = [
                ('name', name),
                ('resource_type', resource_type),
                ('resource', resource_id)
            ]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            self.assertRaises(exceptions.NotFound,
                              self.cmd.take_action,
                              parsed_args)
            self.neutronclient.find_resource.assert_called_with(
                resource_type,
                resource_id,
                cmd_resource='fwaas_firewall_group')
            self.mocked.assert_not_called()

    def test_create_with_invalid_resource_type(self):
        name = self.res['name']
        resource_type = 'invalid_resource_type'
        resource_id = 'valid_fwg_id'
        with mock.patch.object(self.neutronclient, 'find_resource',
                               side_effect=exceptions.NotFound(code=0)):
            arglist = [name,
                       '--resource-type', resource_type,
                       '--resource', resource_id
                       ]
            verifylist = [
                ('name', name),
                ('resource_type', resource_type),
                ('resource', resource_id)
            ]
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            self.assertRaises(exceptions.NotFound,
                              self.cmd.take_action,
                              parsed_args)
            self.neutronclient.find_resource.assert_called_with(
                resource_type,
                resource_id,
                cmd_resource=None)
            self.mocked.assert_not_called()


class TestListNetworkLog(TestNetworkLog):

    def _setup_summary(self, expect=None):
        event = 'Event: ' + self.res['event'].upper()
        target = 'Logged: (None specified)'
        if expect:
            if expect.get('event'):
                event = expect['event']
            if expect.get('resource'):
                target = expect['resource']
        summary = ',\n'.join([event, target])
        self.short_data = (
            expect['id'] if expect else self.res['id'],
            expect['enabled'] if expect else self.res['enabled'],
            expect['name'] if expect else self.res['name'],
            expect['resource_type'] if expect else self.res['resource_type'],
            summary
        )

    def setUp(self):
        super(TestListNetworkLog, self).setUp()
        self.cmd = network_log.ListNetworkLog(self.app, self.namespace)

        self.short_header = (
            'ID',
            'Enabled',
            'Name',
            'Type',
            'Summary',
        )
        self._setup_summary()
        self.neutronclient.list_network_logs = mock.Mock(
            return_value={'logs': [self.res]})
        self.mocked = self.neutronclient.list_network_logs

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.headers), headers)
        self.assertEqual([self.data], list(data))

    def test_list_with_no_option(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.short_header), headers)
        self.assertEqual([self.short_data], list(data))

    def test_list_with_target_and_resource(self):
        arglist = []
        verifylist = []
        target_id = 'aaaaaaaa-aaaa-aaaa-aaaaaaaaaaaaaaaaa'
        resource_id = 'bbbbbbbb-bbbb-bbbb-bbbbbbbbbbbbbbbbb'
        log = fakes.NetworkLog().create({
            'target_id': target_id,
            'resource_id': resource_id})
        self.mocked.return_value = {'logs': [log]}
        logged = 'Logged: (security_group) %(res_id)s on (port) %(t_id)s' % {
            'res_id': resource_id, 't_id': target_id}
        expect_log = copy.deepcopy(log)
        expect_log.update({
            'resource': logged,
            'event': 'Event: ALL'})
        self._setup_summary(expect=expect_log)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.short_header), headers)
        self.assertEqual([self.short_data], list(data))

    def test_list_with_resource(self):
        arglist = []
        verifylist = []
        resource_id = 'bbbbbbbb-bbbb-bbbb-bbbbbbbbbbbbbbbbb'
        log = fakes.NetworkLog().create({'resource_id': resource_id})
        self.mocked.return_value = {'logs': [log]}
        logged = 'Logged: (security_group) %s' % resource_id
        expect_log = copy.deepcopy(log)
        expect_log.update({
            'resource': logged,
            'event': 'Event: ALL'})
        self._setup_summary(expect=expect_log)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.short_header), headers)
        self.assertEqual([self.short_data], list(data))

    def test_list_with_target(self):
        arglist = []
        verifylist = []
        target_id = 'aaaaaaaa-aaaa-aaaa-aaaaaaaaaaaaaaaaa'
        log = fakes.NetworkLog().create({'target_id': target_id})
        self.mocked.return_value = {'logs': [log]}
        logged = 'Logged: (port) %s' % target_id
        expect_log = copy.deepcopy(log)
        expect_log.update({
            'resource': logged,
            'event': 'Event: ALL'})
        self._setup_summary(expect=expect_log)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.short_header), headers)
        self.assertEqual([self.short_data], list(data))


class TestShowNetworkLog(TestNetworkLog):

    def setUp(self):
        super(TestShowNetworkLog, self).setUp()
        self.neutronclient.show_network_log = mock.Mock(
            return_value={'log': self.res})
        self.mocked = self.neutronclient.show_network_log
        self.cmd = network_log.ShowNetworkLog(self.app, self.namespace)

    def test_show_filtered_by_id_or_name(self):
        target = self.res['id']
        arglist = [target]
        verifylist = [('network_log', target)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target)
        self.assertEqual(self.ordered_headers, headers)
        self.assertEqual(self.ordered_data, data)


class TestSetNetworkLog(TestNetworkLog):

    def setUp(self):
        super(TestSetNetworkLog, self).setUp()
        self.neutronclient.update_network_log = mock.Mock(
            return_value={'log': self.res})
        self.mocked = self.neutronclient.update_network_log
        self.cmd = network_log.SetNetworkLog(self.app, self.namespace)

    def test_set_name(self):
        target = self.res['id']
        update = 'change'
        arglist = [target, '--name', update]
        verifylist = [
            ('network_log', target),
            ('name', update),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {'log': {'name': update}})
        self.assertIsNone(result)

    def test_set_description(self):
        target = self.res['id']
        update = 'change-desc'
        arglist = [target, '--description', update]
        verifylist = [
            ('network_log', target),
            ('description', update),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {'log': {'description': update}})
        self.assertIsNone(result)

    def test_set_enable(self):
        target = self.res['id']
        arglist = [target, '--enable']
        verifylist = [
            ('network_log', target),
            ('enable', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {'log': {'enabled': True}})
        self.assertIsNone(result)

    def test_set_disable(self):
        target = self.res['id']
        arglist = [target, '--disable']
        verifylist = [
            ('network_log', target),
            ('disable', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(
            target, {'log': {'enabled': False}})
        self.assertIsNone(result)

    # Illegal tests
    def test_illegal_set_resource_type(self):
        target = self.res['id']
        resource_type = 'security_group'
        arglist = [target, '--resource-type', resource_type]
        verifylist = [
            ('network_log', target),
            ('resource_type', resource_type),
        ]

        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_illegal_set_event(self):
        target = self.res['id']
        for event in ['all', 'accept', 'drop']:
            arglist = [target, '--event', event]
            verifylist = [
                ('network_log', target),
                ('event', event),
            ]
            self.assertRaises(
                utils.ParserException,
                self.check_parser, self.cmd, arglist, verifylist)

    def test_illegal_set_resource_id(self):
        target = self.res['id']
        resource_id = 'resource-id-for-logged-target'
        arglist = [target, '--resource', resource_id]
        verifylist = [
            ('network_log', target),
            ('resource', resource_id),
        ]

        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_illegal_set_project(self):
        target = self.res['id']
        arglist = [
            target,
            '--project',
        ]
        verifylist = [
            ('network_log', target),
            ('project', 'other-project'),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_illegal_set_project_domain(self):
        target = self.res['id']
        arglist = [
            target,
            '--project-domain',
        ]
        verifylist = [
            ('network_log', target),
            ('project_domain', 'other-project-domain'),
        ]
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_illegal_set_and_raises(self):
        self.neutronclient.update_network_log = mock.Mock(
            side_effect=Exception)
        target = self.res['id']
        arglist = [target, '--name', 'my-name']
        verifylist = [('network_log', target), ('name', 'my-name')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestDeleteNetworkLog(TestNetworkLog):

    def setUp(self):
        super(TestDeleteNetworkLog, self).setUp()
        self.neutronclient.delete_network_log = mock.Mock(
            return_value={'log': self.res})
        self.mocked = self.neutronclient.delete_network_log
        self.cmd = network_log.DeleteNetworkLog(self.app, self.namespace)

    def test_delete_with_one_resource(self):
        target = self.res['id']
        arglist = [target]
        verifylist = [('network_log', [target])]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with(target)
        self.assertIsNone(result)

    def test_delete_with_multiple_resources(self):
        target1 = 'target1'
        target2 = 'target2'
        arglist = [target1, target2]
        verifylist = [('network_log', [target1, target2])]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)

        self.assertEqual(2, self.mocked.call_count)
        for idx, reference in enumerate([target1, target2]):
            actual = ''.join(self.mocked.call_args_list[idx][0])
            self.assertEqual(reference, actual)

    def test_delete_with_no_exist_id(self):
        self.neutronclient.find_resource.side_effect = Exception
        target = 'not_exist'
        arglist = [target]
        verifylist = [('network_log', [target])]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestLoggableResource(test_fakes.TestNeutronClientOSCV2):

    def check_results(self, headers, data, exp_req, is_list=False):
        if is_list:
            req_body = {'logs': [exp_req]}
        else:
            req_body = {'log': exp_req}
        self.mocked.assert_called_once_with(req_body)
        self.assertEqual(self.ordered_headers, headers)
        self.assertEqual(self.ordered_data, data)

    def setUp(self):
        super(TestLoggableResource, self).setUp()
        self.headers = ('Supported types',)
        self.data = ('security_group', )


class TestListLoggableResource(TestLoggableResource):

    def setUp(self):
        super(TestListLoggableResource, self).setUp()
        self.cmd = network_log.ListLoggableResource(self.app, self.namespace)

        loggables = {
            "loggable_resources": [{"type": "security_group"}]
        }
        self.neutronclient.list_network_loggable_resources = mock.Mock(
            return_value=loggables)
        self.mocked = self.neutronclient.list_network_loggable_resources

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.headers), headers)
        self.assertEqual([self.data], list(data))

    def test_list_with_no_option(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        headers, data = self.cmd.take_action(parsed_args)

        self.mocked.assert_called_once_with()
        self.assertEqual(list(self.headers), headers)
        self.assertEqual([self.data], list(data))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient.tests.functional import base


class SimpleReadOnlyNeutronFwv1ClientTest(base.ClientTestBase):

    """Tests for FWaaS v1 based client commands that are read only"""

    def setUp(self):
        super(SimpleReadOnlyNeutronFwv1ClientTest, self).setUp()
        if not self.is_extension_enabled('fwaas'):
            self.skipTest('FWaaS is not enabled')

    def test_neutron_firewall_list(self):
        firewall_list = self.parser.listing(self.neutron
                                            ('firewall-list'))
        self.assertTableStruct(firewall_list, ['id', 'name',
                                               'firewall_policy_id'])

    def test_neutron_firewall_policy_list(self):
        firewall_policy = self.parser.listing(self.neutron
                                              ('firewall-policy-list'))
        self.assertTableStruct(firewall_policy, ['id', 'name',
                                                 'firewall_rules'])

    def test_neutron_firewall_rule_list(self):
        firewall_rule = self.parser.listing(self.neutron
                                            ('firewall-rule-list'))
        self.assertTableStruct(firewall_rule, ['id', 'name',
                                               'firewall_policy_id',
                                               'summary', 'enabled'])
